/**
 * ServiceLocationAttribute.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;

import org.sblim.slp.internal.Utils;

/**
 * Service location attribute
 */
public class ServiceLocationAttribute implements Serializable {

	private static final long serialVersionUID = -6753246108754657715L;

	private Vector iValues;

	private String iId;

	/**
	 * Construct a service location attribute. Errors in the id or values vector
	 * result in an IllegalArgumentException.
	 * 
	 * @param pId
	 *            The attribute name. The String can consist of any Unicode
	 *            character.
	 * @param pValues
	 *            A Vector of one or more attribute values. Vector contents must
	 *            be uniform in type and one of Integer, String, Boolean, or
	 *            byte[]. If the attribute is a keyword attribute, then the
	 *            parameter should be null. String values can consist of any
	 *            Unicode character.
	 */
	public ServiceLocationAttribute(String pId, Vector pValues) {
		iId = pId;
		if (pValues != null && pValues.size() > 0) {
			iValues = (Vector) pValues.clone();
		}
	}

	/**
	 * Construct a service location attribute from a String.
	 * 
	 * @param pString
	 *            The string to parse
	 * @throws ServiceLocationException
	 *             When the string parsing failed
	 */
	public ServiceLocationAttribute(String pString) throws ServiceLocationException {
		iValues = null;
		iId = null;
		if (pString == null || pString.length() <= 0) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR);

		if (pString.startsWith("(") && pString.endsWith(")")) {
			try {
				int equalPos = pString.indexOf('=');

				String valueString = "";
				if (equalPos == -1) {
					equalPos = pString.length();
					iId = Utils.unescapeAttributeString(pString.substring(1, pString.length() - 1));
				} else {
					iId = Utils.unescapeAttributeString(pString.substring(1, equalPos));

					valueString = pString.substring(equalPos + 1, pString.length() - 1);
				}
				if (iId.length() <= 0) throw new ServiceLocationException(
						ServiceLocationException.PARSE_ERROR);

				iValues = Utils.parseCommaSeparatedList(valueString, true);

				int j = iValues.size();
				for (int i = 0; i < j; i++) {
					iValues.setElementAt(parseObjectFromString((String) iValues.elementAt(i)), i);
				}

			} catch (NoSuchElementException e) {
				throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR, e);
			}
		} else {
			if (pString.indexOf('(') != -1 || pString.indexOf(')') != -1) throw new ServiceLocationException(
					ServiceLocationException.PARSE_ERROR);
			iId = Utils.unescapeAttributeString(pString);
		}
	}

	private static Object parseObjectFromString(String pString) throws ServiceLocationException {
		Object obj = null;
		try {
			obj = Integer.valueOf(pString);
		} catch (NumberFormatException e) {

			if ("true".equalsIgnoreCase(pString) || "false".equalsIgnoreCase(pString)) {
				obj = Boolean.valueOf(pString);
			} else {
				if (pString.startsWith("\\ff")) obj = new Opaque(Utils.unescapeByteArray(pString));
				else obj = Utils.unescapeAttributeString(pString);
			}
		}
		return obj;
	}

	/**
	 * Returns an escaped version of the id parameter, suitable for inclusion in
	 * a query. Any reserved characters as specified in [7] are escaped using
	 * UTF-8 encoding. If any characters in the tag are illegal, throws
	 * IllegalArgumentException.
	 * 
	 * @param pId
	 *            The attribute id to escape. ServiceLocationException is thrown
	 *            if any characters are illegal for an attribute tag.
	 * @return The escaped version
	 */
	public static String escapeId(String pId) {
		try {
			return Utils.escapeAttributeString(pId);
		} catch (ServiceLocationException e) {
			// TODO Auto-generated catch block
			throw new IllegalArgumentException();
		}
	}

	/**
	 * Returns a String containing the escaped value parameter as a string,
	 * suitable for inclusion in a query. If the parameter is a string, any
	 * reserved characters as specified in [7] are escaped using UTF-8 encoding.
	 * If the parameter is a byte array, then the escaped string begins with the
	 * nonUTF-8 sequence `\ff` and the rest of the string consists of the
	 * escaped bytes, which is the encoding for opaques. If the value parameter
	 * is a Boolean or Integer, then the returned string contains the object
	 * converted into a string. If the value is any type other than String,
	 * Integer, Boolean or byte[], an IllegalArgumentException is thrown.
	 * 
	 * @param pValue
	 *            The attribute value to be converted into a string and escaped.
	 * @return The escaped value
	 */
	public static String escapeValue(Object pValue) {
		if (pValue == null) throw new IllegalArgumentException("null value");

		if ("".equals(pValue)) throw new IllegalArgumentException("empty string");

		if (!(pValue instanceof Integer) && !(pValue instanceof Boolean)
				&& !(pValue instanceof String) && !(pValue instanceof byte[])) throw new IllegalArgumentException(
				"invalid type");

		if (pValue instanceof byte[]) pValue = new Opaque((byte[]) pValue); // just
																			// in
																			// case

		String s;
		if (pValue instanceof String) {
			try {
				s = Utils.escapeAttributeString((String) pValue);
			} catch (ServiceLocationException e) {
				throw new IllegalArgumentException();
			}
		} else s = pValue.toString();
		return s;
	}

	/**
	 * Returns a cloned vector of attribute values, or null if the attribute is
	 * a keyword attribute. If the attribute is single-valued, then the vector
	 * contains only one object.
	 * 
	 * @return The value vector
	 * 
	 */
	public Vector getValues() {
		if (iValues != null) return (Vector) iValues.clone();
		return iValues;
	}

	/**
	 * Returns the attribute's name.
	 * 
	 * @return The name (id)
	 */
	public String getId() {
		return iId;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 * 
	 * Overrides Object.equals(). Two attributes are equal if their identifiers
	 * are equal and their value vectors contain the same number of equal values
	 * as determined by the Object equals() method. Values having byte[] type
	 * are equal if the contents of all byte arrays in both attribute vectors
	 * match. Note that the SLP string matching algorithm [7] MUST NOT be used
	 * for comparing attribute identifiers or string values.
	 */
	public boolean equals(Object obj) {
		if (!(obj instanceof ServiceLocationAttribute)) return false;
		if (obj == this) return true;

		ServiceLocationAttribute that = (ServiceLocationAttribute) obj;
		if (!that.getId().equalsIgnoreCase(iId)) return false;
		if (iValues == null && that.iValues == null) return true;
		if (iValues == null && that.iValues != null || iValues != null && that.iValues == null) return false;

		if (iValues.size() != that.iValues.size()) return false;

		if (iValues.size() > 0
				&& that.iValues.elementAt(0).getClass() != iValues.elementAt(0).getClass()) { return false; }

		int size = that.iValues.size();
		for (int i = 0; i < size; i++) {
			if (!iValues.contains(that.iValues.elementAt(i))) return false;
		}

		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 * 
	 * Overrides Object.toString(). The string returned contains a formatted
	 * representation of the attribute, giving the attribute's id, values, and
	 * the Java type of the values. The returned string is suitable for
	 * debugging purposes, but is not in SLP wire format.
	 */
	public String toString() {
		StringBuffer stringbuffer = new StringBuffer("(");
		stringbuffer.append(iId);
		if (iValues != null) {
			stringbuffer.append("=");
			int size = iValues.size();
			for (int i = 0; i < size; i++) {
				Object obj = iValues.elementAt(i);
				if (i > 0) {
					stringbuffer.append(",");
				}
				stringbuffer.append(obj.toString());
			}

		}
		stringbuffer.append(")");
		return stringbuffer.toString();
	}

	/**
	 * Returns the attribute serialized (as comma separated list)
	 * 
	 * @return The serialized attribute
	 * @throws ServiceLocationException
	 */
	public String serialize() throws ServiceLocationException {

		if (iValues == null) return Utils.escapeAttributeString(iId);

		Vector v = new Vector();
		String s;
		for (Enumeration enumeration = iValues.elements(); enumeration.hasMoreElements(); v
				.addElement(s)) {
			Object obj = enumeration.nextElement();
			s = null;
			if (obj instanceof String) try {
				s = Utils.escapeAttributeString((String) obj);
			} catch (ServiceLocationException e) {
				s = obj.toString();
			}
			else s.toString();
		}

		StringBuffer sb = new StringBuffer("(" + Utils.escapeAttributeString(iId) + "=");
		sb.append(Utils.vectorToCommaSeparatedList(v));
		sb.append(")");
		return sb.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 * 
	 * Overrides Object.hashCode(). Hashes on the attribute's identifier.
	 */
	public int hashCode() {
		return ((iValues != null) ? iValues.hashCode() : 0) * 37
				+ ((iId != null) ? iId.hashCode() : 0);
	}

	public static void main(String[] args) {}
}
