/**
 * ServiceLocationException.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 */

package org.sblim.slp;

/**
 * The ServiceLocationException class is thrown by all methods when exceptional
 * conditions occur in the SLP framework. The error code property determines the
 * exact nature of the condition, and an optional message may provide more
 * information.
 */
public class ServiceLocationException extends RuntimeException {

	private static final long serialVersionUID = 6414188770647750627L;

	public static final short OK = 0;

	public static final short LANGUAGE_NOT_SUPPORTED = 1;

	public static final short PARSE_ERROR = 2;

	public static final short INVALID_REGISTRATION = 3;

	public static final short SCOPE_NOT_SUPPORTED = 4;

	public static final short AUTHENTICATION_UNKNOWN = 5;

	public static final short AUTHENTICATION_ABSENT = 6;

	public static final short AUTHENTICATION_FAILED = 7;

	public static final short VERSION_NOT_SUPPORTED = 9;

	public static final short INTERNAL_ERROR = 10;

	public static final short DA_BUSY = 11;

	public static final short OPTION_NOT_SUPPORTED = 12;

	public static final short INVALID_UPDATE = 13;

	public static final short REQUEST_NOT_SUPPORTED = 14;

	public static final short REFRESH_REJECTED = 15;

	public static final short NOT_IMPLEMENTED = 16;

	public static final short NETWORK_INIT_FAILED = 17;

	public static final short NETWORK_TIMED_OUT = 18;

	public static final short NETWORK_ERROR = 19;

	public static final short INTERNAL_SYSTEM_ERROR = 20;

	public static final short TYPE_ERROR = 21;

	public static final short BUFFER_OVERFLOW = 22;

	public static final short PREVIOUS_RESPONDER_OVERFLOW = 100;

	private static final String[] ERROR_MESSAGES = { "OK", "LANGUAGE_NOT_SUPPORTED", "PARSE_ERROR",
			"INVALID_REGISTRATION", "SCOPE_NOT_SUPPORTED", "AUTHENTICATION_UNKNOWN",
			"AUTHENTICATION_ABSENT", "AUTHENTICATION_FAILED", "", "VERSION_NOT_SUPPORTED",
			"INTERNAL_ERROR", "DA_BUSY", "OPTION_NOT_SUPPORTED", "INVALID_UPDATE",
			"REQUEST_NOT_SUPPORTED", "REFRESH_REJECTED", "NOT_IMPLEMENTED", "NETWORK_INIT_FAILED",
			"NETWORK_TIMED_OUT", "NETWORK_ERROR", "INTERNAL_SYSTEM_ERROR", "TYPE_ERROR",
			"BUFFER_OVERFLOW" };

	short iErrorCode;

	private Object iExtendedReason[];

	/**
	 * Ctor.
	 * 
	 * @param pErrorCode
	 *            One of the ec constants in this class
	 */
	public ServiceLocationException(short pErrorCode) {
		super(ERROR_MESSAGES[pErrorCode]);
		iErrorCode = pErrorCode;
		iExtendedReason = null;
	}

	/**
	 * Ctor.
	 * 
	 * @param pErrorCode
	 *            One of the ec constants in this class
	 * @param pCause
	 *            The cause
	 */
	public ServiceLocationException(short pErrorCode, Throwable pCause) {
		super(ERROR_MESSAGES[pErrorCode], pCause);
		iErrorCode = pErrorCode;
		iExtendedReason = null;
	}

	/**
	 * Ctor.
	 * 
	 * @param pErrorCode
	 *            One of the ec constants in this class
	 * @param pMessage
	 *            A more specific message
	 * @param pExtendedReason
	 *            Extended reason
	 */
	public ServiceLocationException(short pErrorCode, String pMessage, Object pExtendedReason[]) {
		super(ERROR_MESSAGES[pErrorCode] + "(" + pMessage + ")");
		iErrorCode = pErrorCode;
		iExtendedReason = pExtendedReason;
	}

	/**
	 * Ctor.
	 * 
	 * @param pErrorCode
	 *            One of the ec constants in this class
	 * @param pMessage
	 *            A more specific message
	 */
	public ServiceLocationException(short pErrorCode, String pMessage) {
		super(ERROR_MESSAGES[pErrorCode] + "(" + pMessage + ")");
		iErrorCode = pErrorCode;
		iExtendedReason = null;
	}

	/**
	 * Ctor.
	 * 
	 * @param pErrorCode
	 *            One of the ec constants in this class
	 * @param pMessage
	 *            A more specific message
	 * @param pExtendedReason
	 *            Extended reason
	 */
	public ServiceLocationException(short pErrorCode, String pMessage, Object pExtendedReason) {
		super(ERROR_MESSAGES[pErrorCode] + "(" + pMessage + ")");
		iErrorCode = pErrorCode;
		iExtendedReason = new Object[] { pExtendedReason };
	}

	/**
	 * Return the error code. The error code takes on one of the static field
	 * values.
	 * 
	 * @return The error code
	 */
	public short getErrorCode() {
		return iErrorCode;
	}

	/**
	 * Specifies the extended reason for this exception.
	 * 
	 * @param pExtendedReason
	 */
	public void setExtendedReason(Object[] pExtendedReason) {
		iExtendedReason = pExtendedReason;
	}

	/**
	 * Gets the extended reason for this exception.
	 * 
	 * @return The extended reason
	 */
	public Object[] getExtendedReason() {
		return iExtendedReason;
	}

	/**
	 * Gets the message associated to this exception.
	 * 
	 * @return The message
	 */
	public String getMessage() {
		if (getCause() == null) { return super.getMessage(); }
		return super.getMessage() + "; nested exception is: \n\t" + getCause().toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if (iExtendedReason != null) {
			StringBuffer sb = new StringBuffer();
			int i = 0;
			// boolean anyExtendedReason = false;
			sb.append('(');
			for (; i < iExtendedReason.length - 1; i++) {
				if (iExtendedReason[i] != null) {
					sb.append(iExtendedReason[i].toString());
				} else {
					sb.append("null");
				}
				sb.append(',');
			}
			sb.append(iExtendedReason[i]);
			sb.append(')');
			sb.insert(0, super.toString());
			return sb.toString();
		}
		return super.toString();
	}
}
