/**
 * ServiceURL.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp;

import java.io.Serializable;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

/**
 * The ServiceURL object models the advertised SLP service URL. It can be either
 * a service: URL or a regular URL. These objects are returned from service
 * lookup requests, and describe the registered services. This class should be a
 * subclass of java.net.URL but can't since that class is final.
 */
public class ServiceURL implements Serializable {

	private static final long serialVersionUID = 8998115518853094365L;

	/**
	 * Indicates that no port information is required or was returned for this
	 * URL.
	 */
	public static final int NO_PORT = 0;

	/**
	 * Indicates that the URL has a zero lifetime. This value is never returned
	 * from the API, but can be used to create a ServiceURL object to
	 * deregister, delete attributes, or find attributes.
	 */
	public static final int LIFETIME_NONE = 0;

	/**
	 * The default URL lifetime (3 hours) in seconds.
	 */
	public static final int LIFETIME_DEFAULT = 10800;

	/**
	 * The maximum URL lifetime (about 18 hours) in seconds.
	 */
	public static final int LIFETIME_MAXIMUM = 65535;

	/**
	 * Indicates that the API implementation should continuously re-register the
	 * URL until the application exits.
	 */
	public static final int LIFETIME_PERMANENT = -1;

	static final int PORT_MAXIMUM = 65535;

	private ServiceType iServiceType = null;

	private ServiceType iOriginalServiceType = null;

	private String iTransport = "";

	private String iHost = "";

	private int iPort = 0;

	private String iURLPath = "";

	private int iLifetime = LIFETIME_DEFAULT;

	private boolean iNoDoubleSlash = false;

	/**
	 * Construct a service URL object having the specified lifetime.
	 * 
	 * @param pServiceURL
	 *            The URL as a string. Must be either a service: URL or a valid
	 *            generic URL according to RFC 2396 [2].
	 * @param pLifetime
	 *            The service advertisement lifetime in seconds. This value may
	 *            be either between LIFETIME_NONE and LIFETIME_MAXIMUM or
	 *            LIFETIME_PERMANENT.
	 */
	public ServiceURL(String pServiceURL, int pLifetime) {

		if (pLifetime > LIFETIME_MAXIMUM || pLifetime < LIFETIME_PERMANENT) throw new IllegalArgumentException(
				"lifetime:" + pLifetime);

		for (int i = 0; i < pServiceURL.length(); i++) {
			char c = pServiceURL.charAt(i);
			if ("/:-.%_\'*()$!,+\\;@?&=".indexOf(c) == -1 && !Character.isLetterOrDigit(c)) { throw new IllegalArgumentException(
					"invalid character: '" + c + "' on string \"" + pServiceURL + "\""); }
		}

		parseURL(pServiceURL);

		iLifetime = (pLifetime == LIFETIME_PERMANENT) ? LIFETIME_MAXIMUM : pLifetime;
	}

	/**
	 * Returns the service type object representing the service type name of the
	 * URL.
	 * 
	 * @return The service type
	 */
	public ServiceType getServiceType() {
		return iServiceType;
	}

	/**
	 * Set the service type name to the object. Ignored if the URL is a service:
	 * URL.
	 * 
	 * @param pServicetype
	 *            The service type object.
	 * @throws ServiceLocationException
	 */
	public final void setServiceType(ServiceType pServicetype) throws ServiceLocationException {
		if (!iServiceType.isServiceURL()) iServiceType = pServicetype;
	}

	/**
	 * Get the network layer transport identifier. If the transport is IP, an
	 * empty string, "", is returned.
	 * 
	 * @return The NLT identifier
	 */
	public String getTransport() {
		return iTransport;
	}

	/**
	 * Returns the host identifier. For IP, this will be the machine name or IP
	 * address.
	 * 
	 * @return The host
	 */
	public String getHost() {
		return iHost;
	}

	/**
	 * Returns the port number, if any. For non-IP transports, always returns
	 * NO_PORT.
	 * 
	 * @return The port
	 */
	public int getPort() {
		return iPort;
	}

	/**
	 * Returns the URL path description, if any.
	 * 
	 * @return The URL path
	 */
	public String getURLPath() {
		return iURLPath;
	}

	/**
	 * Returns the service advertisement lifetime. This will be a positive int
	 * between LIFETIME_NONE and LIFETIME_MAXIMUM.
	 * 
	 * @return The lifetime
	 */
	public int getLifetime() {
		return iLifetime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 * 
	 * Compares the object to the ServiceURL and returns true if the two are the
	 * same. Two ServiceURL objects are equal if their current service types
	 * match and they have the same host, port, transport, and URL path.
	 */
	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (!(obj instanceof ServiceURL)) return false;

		ServiceURL that = (ServiceURL) obj;
		return this.iServiceType.equals(that.iServiceType)
				&& this.iTransport.equals(that.iTransport) && this.iHost.equals(that.iHost)
				&& this.iPort == that.iPort && this.iNoDoubleSlash == that.iNoDoubleSlash
				&& this.iURLPath.equals(that.iURLPath);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 * 
	 * Returns a formatted string with the URL. Overrides Object.toString(). The
	 * returned URL has the original service type or URL scheme, not the current
	 * service type.
	 */
	public String toString() {
		return iOriginalServiceType.toString() + ":/" + iTransport + (iNoDoubleSlash ? "" : "/")
				+ iHost + (iPort == 0 ? "" : ":" + iPort) + iURLPath;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 * 
	 * Overrides Object.hashCode(). Hashes on the current service type,
	 * transport, host, port, and URL part.
	 */
	public int hashCode() {
		return iServiceType.hashCode() + iTransport.hashCode() + iHost.hashCode() + iPort
				+ iURLPath.hashCode();
	}

	/**
	 * Parses a URL string and updates all members accordingly. "service:"<srvtype>"://"<addrspec>
	 * 
	 * @param pUrlString
	 *            The URL string
	 */
	private void parseURL(String pUrlString) throws IllegalArgumentException {
		while (true) {
			StringTokenizer st = new StringTokenizer(pUrlString, "/", true);
			try {

				String servicetypeToken = st.nextToken();
				if (servicetypeToken.equals("/") || !servicetypeToken.endsWith(":")) continue;

				iServiceType = new ServiceType(servicetypeToken.substring(0, servicetypeToken
						.length() - 1));

				iOriginalServiceType = iServiceType;
				String slashToken = st.nextToken();
				if (!slashToken.equals("/")) continue;

				String s3 = st.nextToken();
				String hstToken = "";
				if (!s3.equals("/")) {
					if (!iServiceType.isServiceURL()) {
						hstToken = s3;
						iNoDoubleSlash = true;
					} else if (!s3.equalsIgnoreCase("ipx") && !s3.equalsIgnoreCase("at")) {
						if (!iServiceType.isAbstractType()) continue;

						hstToken = s3;
						iNoDoubleSlash = true;
					} else {
						iTransport = s3.toLowerCase();
						if (!st.nextToken().equals("/")) continue;
						hstToken = st.nextToken();
					}
				} else {
					hstToken = st.nextToken();
				}

				if (hstToken.equals("/")) { // using double slash
					iURLPath = "/" + st.nextToken("");
					return;
				}
				iHost = hstToken;
				if (iTransport.equals("")) {
					StringTokenizer hostST = new StringTokenizer(iHost, ":");
					iHost = hostST.nextToken();
					if (hostST.hasMoreTokens()) {
						String portToken = hostST.nextToken();

						if (hostST.hasMoreTokens()) continue;

						try {
							iPort = Integer.parseInt(portToken);
						} catch (NumberFormatException e) {
							continue;
						}
						if (iPort <= 0 || iPort > 65535) continue;
					}
				}

				if (!st.hasMoreTokens()) return;

				String urlPathTocken = st.nextToken();

				if (urlPathTocken.equals("/")) {
					iURLPath = urlPathTocken;
					if (st.hasMoreTokens()) iURLPath += st.nextToken("");
					iURLPath = iURLPath.trim();
					return;
				}
			} catch (NoSuchElementException e) {
				throw new IllegalArgumentException("Error while parsing URL:" + pUrlString);
			}
			throw new IllegalArgumentException("Error while parsing URL:" + pUrlString);
		}
	}

	public static void main(String[] args) {
		ServiceURL service = new ServiceURL("service:wbem:http://localhost:5988/cimom", -1);
		System.out.println(service);
	}
}
