/**
 * LocatorImpl.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 16966      2005-04-29  thschaef     Add findServices() signature with DAs as parameter
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.Locator;
import org.sblim.slp.ServiceLocationAttribute;
import org.sblim.slp.ServiceLocationEnumeration;
import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

/**
 * The class LocatorImpl implements the Locator interface as defined by RFC 3224
 */
public class LocatorImpl implements Locator {

	private SLPConfig iConfig;

	private Locale iLocale;

	LocatorImpl(Locale pLocale) {
		if (iConfig == null) iConfig = SLPConfig.getGlobalSLPConfig();

		iLocale = pLocale;
	}

	// 16966
	private Vector createMessages(int pFunctionCode, String pQueryStr, String pNamingAuthority,
			ServiceType pServicetype, Vector pScopes, Vector pAttributeIds, Vector pDiscoveryAgents)
			throws ServiceLocationException {

		// Validate scopes
		// validateScopes(vector, locale);
		ServiceLocationMessage request = null;

		// TODO get the list of DA from the config, and the Active DA discovery
		// engine
		Vector daAddresses = null;

		// 16966
		if (pDiscoveryAgents == null) {
			daAddresses = SLPConfig.getGlobalSLPConfig().getPreconfiguredDAs();
		} else {
			daAddresses = pDiscoveryAgents;
		}

		Vector saScopes = new Vector();
		// TODO filter DA using scopes
		saScopes.addAll(pScopes);

		if ((pFunctionCode == ServiceLocationHeader.SRVRQST || pFunctionCode == ServiceLocationHeader.ATTRRQST)

				&& (pServicetype.equals(SLPDefaults.DA_SERVICE_TYPE) || pServicetype
						.equals(SLPDefaults.SA_SERVICE_TYPE))) {

			String query = "";
			if (pFunctionCode == ServiceLocationHeader.SRVRQST && pQueryStr != null) query = pQueryStr;

			request = new SLPServiceRequestMessage(iLocale, pServicetype, pScopes, query);
		} else {
			switch (pFunctionCode) {
				case ServiceLocationHeader.SRVRPLY:
				case ServiceLocationHeader.SRVREG:
				case ServiceLocationHeader.SRVDEREG:
				case ServiceLocationHeader.SRVACK:
				case ServiceLocationHeader.ATTRRPLY:
				case ServiceLocationHeader.DAADVERT:
				default:
					break;
				case ServiceLocationHeader.SRVTYPERQST:
					request = new SLPServiceTypeRequestMessage(iLocale, pNamingAuthority, pScopes);
					break;

				case ServiceLocationHeader.SRVRQST:
					request = new SLPServiceRequestMessage(iLocale, pServicetype, pScopes,
							pQueryStr);
					break;
				case ServiceLocationHeader.ATTRRQST:
					request = new SLPAttributeRequestMessage(iLocale, pServicetype, pScopes,
							pAttributeIds);
					break;
			}
		}
		return MessageDispatcher.dispatchUAMsg(daAddresses, request, SLPConfig.getMulticastAddress());
	}

	public synchronized ServiceLocationEnumeration findServices(ServiceType pServicetype,
			Vector pScopes, String pQuery) throws ServiceLocationException {

		if (pServicetype == null) throw new IllegalArgumentException("null type");
		if (pScopes == null) throw new IllegalArgumentException("null scopes");
		if (pQuery == null) throw new IllegalArgumentException("null query");

		Vector servicesResponse = createMessages(1, pQuery, null, pServicetype, pScopes, null, null);
		Vector result = new Vector();

		int maxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

		int totalServices = servicesResponse.size();
		for (int i = 0; i < totalServices; i++) {
			ServiceLocationMessage srvlocmsg = (ServiceLocationMessage) servicesResponse
					.elementAt(i);
			short n;
			if ((n = srvlocmsg.getErrorCode()) != 0 && servicesResponse.size() == 0) throw new ServiceLocationException(
					n);

			Vector urlList = null;

			if (srvlocmsg instanceof SLPServiceReplyMessage) {
				urlList = ((SLPServiceReplyMessage) srvlocmsg).getServiceURLs();
			} else if (srvlocmsg instanceof SLPSAAdvert) {
				urlList = new Vector();
				urlList.addElement(((SLPSAAdvert) srvlocmsg).getUrl());
			} else if (srvlocmsg instanceof SLPDAAdvert) {
				urlList = new Vector();
				urlList.addElement(((SLPDAAdvert) srvlocmsg).getUrl());
			}

			if (urlList != null) {
				int size = urlList.size();
				for (int m = 0; m < size; m++) {
					Object o = urlList.elementAt(m);

					if (!result.contains(o) && result.size() < maxResults) result.addElement(o);
				}
			}
		}

		return new ServiceLocationEnumerator(result);
	}

	public synchronized ServiceLocationEnumeration findServices(ServiceType pServicetype,
			Vector pScopes, String pQuery, Vector pDiscoveryAgents) throws ServiceLocationException {

		if (pServicetype == null) throw new IllegalArgumentException("null type");
		if (pScopes == null) throw new IllegalArgumentException("null scopes");
		if (pQuery == null) throw new IllegalArgumentException("null query");

		Vector servicesResponse = createMessages(1, pQuery, null, pServicetype, pScopes, null,
				pDiscoveryAgents);
		Vector result = new Vector();

		int maxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

		int totalServices = servicesResponse.size();
		for (int i = 0; i < totalServices; i++) {
			ServiceLocationMessage srvlocmsg = (ServiceLocationMessage) servicesResponse
					.elementAt(i);
			short n;
			if ((n = srvlocmsg.getErrorCode()) != 0 && servicesResponse.size() == 0) throw new ServiceLocationException(
					n);

			Vector urlList = null;

			if (srvlocmsg instanceof SLPServiceReplyMessage) {
				urlList = ((SLPServiceReplyMessage) srvlocmsg).getServiceURLs();
			} else if (srvlocmsg instanceof SLPSAAdvert) {
				urlList = new Vector();
				urlList.addElement(((SLPSAAdvert) srvlocmsg).getUrl());
			} else if (srvlocmsg instanceof SLPDAAdvert) {
				urlList = new Vector();
				urlList.addElement(((SLPDAAdvert) srvlocmsg).getUrl());
			}

			if (urlList != null) {
				int size = urlList.size();
				for (int m = 0; m < size; m++) {
					Object o = urlList.elementAt(m);

					if (!result.contains(o) && result.size() < maxResults) result.addElement(o);
				}
			}
		}

		return new ServiceLocationEnumerator(result);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Locator#getLocale()
	 */
	public Locale getLocale() {
		return iLocale;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Locator#findServiceTypes(java.lang.String,
	 *      java.util.Vector)
	 */
	public ServiceLocationEnumeration findServiceTypes(String pNamingAuthority, Vector pScopes)
			throws ServiceLocationException {
		if (pNamingAuthority == null) throw new IllegalArgumentException("Naming Authority");
		if (pScopes == null) throw new IllegalArgumentException("scopes");

		Vector servicesTypeResponse = createMessages(9, null, pNamingAuthority, null, pScopes,
				null, null);
		Vector result = new Vector();
		int totalResponses = servicesTypeResponse.size();

		int maxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();
		for (int i = 0; i < totalResponses; i++) {
			SLPServiceTypeReplyMessage srvtype = (SLPServiceTypeReplyMessage) servicesTypeResponse
					.elementAt(i);

			short e = srvtype.getErrorCode();
			if (e != 0 && servicesTypeResponse.size() == 0) { throw new ServiceLocationException(e); }
			Vector urlList = srvtype.getServiceTypes();
			if (urlList != null) {
				int size = urlList.size();
				for (int m = 0; m < size; m++) {
					Object o = urlList.elementAt(m);

					if (!result.contains(o) && result.size() < maxResults) result.addElement(o);
				}
			}
		}
		return new ServiceLocationEnumerator(result);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Locator#findAttributes(com.ibm.slp.ServiceURL,
	 *      java.util.Vector, java.util.Vector)
	 */
	public ServiceLocationEnumeration findAttributes(ServiceURL pServiceUrl, Vector pScopes,
			Vector pAttributeIds) throws ServiceLocationException {
		if (pServiceUrl == null) throw new IllegalArgumentException("serviceurl");
		if (pScopes == null) throw new IllegalArgumentException("scopes");
		if (pAttributeIds == null) throw new IllegalArgumentException("attributeIds");

		Vector responseMsg = createMessages(6, null, null, pServiceUrl.getServiceType(), pScopes,
				pAttributeIds, null);
		Vector attrList = new Vector();
		int totalResponses = responseMsg.size();
		int maxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();
		for (int i = 0; i < totalResponses; i++) {
			ServiceLocationMessage srvloc = (ServiceLocationMessage) responseMsg.elementAt(i);

			short e = srvloc.getErrorCode();
			if (e != 0 && responseMsg.size() == 1) throw new ServiceLocationException(e);

			if (srvloc instanceof SLPAttributeReplyMessage) attrList = ((SLPAttributeReplyMessage) srvloc)
					.getAttributeList();
			else if (srvloc instanceof SLPSAAdvert) {
				SLPSAAdvert csaadvert = (SLPSAAdvert) srvloc;
				if (!pServiceUrl.equals(csaadvert.getUrl())) continue;
				attrList = csaadvert.getAttributes();
			} else if (srvloc instanceof SLPDAAdvert) {
				SLPDAAdvert cdaadvert = (SLPDAAdvert) srvloc;
				if (!pServiceUrl.equals(cdaadvert.getUrl())) continue;
				attrList = cdaadvert.getAttributes();
			}
			if (attrList.size() > maxResults) attrList.setSize(maxResults);
			break;
		}

		return new ServiceLocationEnumerator(attrList);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Locator#findAttributes(com.ibm.slp.ServiceType,
	 *      java.util.Vector, java.util.Vector)
	 */
	public ServiceLocationEnumeration findAttributes(ServiceType pServiceType, Vector pScopes,
			Vector pAttributeIds) throws ServiceLocationException {
		if (pServiceType == null) throw new IllegalArgumentException("servicetype");
		if (pScopes == null) throw new IllegalArgumentException("scopes");
		if (pAttributeIds == null) throw new IllegalArgumentException("attributeIds");

		Vector responseMsg = createMessages(6, null, null, pServiceType, pScopes, pAttributeIds,
				null);
		Vector results = new Vector();

		int totalResponses = responseMsg.size();
		int maxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

		for (int i = 0; i < totalResponses && results.size() < maxResults; i++) {
			ServiceLocationMessage srvlocmsg = (ServiceLocationMessage) responseMsg.elementAt(i);

			short e = srvlocmsg.getErrorCode();
			if (e != 0 && responseMsg.size() == 1) { throw new ServiceLocationException(e); }

			Vector attrList = null;

			if (srvlocmsg instanceof SLPAttributeReplyMessage) attrList = ((SLPAttributeReplyMessage) srvlocmsg)
					.getAttributeList();
			else if (srvlocmsg instanceof SLPSAAdvert) attrList = ((SLPSAAdvert) srvlocmsg)
					.getAttributes();
			else if (srvlocmsg instanceof SLPDAAdvert) attrList = ((SLPDAAdvert) srvlocmsg)
					.getAttributes();

			int size = attrList.size();
			for (int l = 0; l < size; l++) {
				ServiceLocationAttribute servicelocationattribute = (ServiceLocationAttribute) attrList
						.elementAt(l);
				// TODO remove duplicated attributes
				results.add(servicelocationattribute);
				if (results.size() >= maxResults) break;
			}

		}

		return new ServiceLocationEnumerator(results);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Locator#findServiceTypes(java.lang.String,
	 *      java.util.Vector, java.util.Vector)
	 */
	public ServiceLocationEnumeration findServiceTypes(String pNamingAuthority, Vector pScopes,
			Vector pDirectoryAgent) throws ServiceLocationException {
		throw new RuntimeException("not implemented");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Locator#findAttributes(com.ibm.slp.ServiceURL,
	 *      java.util.Vector, java.util.Vector, java.util.Vector)
	 */
	public ServiceLocationEnumeration findAttributes(ServiceURL pURL, Vector pScopes,
			Vector pAttributeIds, Vector directoryAgents) throws ServiceLocationException {
		throw new RuntimeException("not implemented");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Locator#findAttributes(com.ibm.slp.ServiceType,
	 *      java.util.Vector, java.util.Vector, java.util.Vector)
	 */
	public ServiceLocationEnumeration findAttributes(ServiceType pType, Vector pScopes,
			Vector pAttributeIds, Vector directoryAgents) throws ServiceLocationException {
		throw new RuntimeException("not implemented");
	}

}
