/**
 * MessageDispatcher.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;

public class MessageDispatcher implements Runnable {

	protected int iMaxResults;

	protected ServiceLocationMessage iMsgOut;

	protected Vector iResponseList;

	protected int[] iTimeoutList;

	protected ServiceLocationException iExErr = null;

	protected InetAddress iAddress;

	boolean iJustFirstResult;

	MessageDispatcher(ServiceLocationMessage pRequest, Vector pResponseList, int[] pTimeoutList,
			int pMaxResults, InetAddress pInetAddress, boolean pJustFirstResult) {

		iMsgOut = pRequest;
		iResponseList = pResponseList;
		iTimeoutList = pTimeoutList;
		iMaxResults = pMaxResults;
		iAddress = pInetAddress;
		iJustFirstResult = pJustFirstResult;
	}

	protected static boolean addPreviousResponder(ServiceLocationMessage pRequest,
			InetAddress pInetAddress) {

		ServiceLocationHeader header = pRequest.getHeader();
		Vector previousResponders = header.getPreviousResponse();
		String localhost = pInetAddress.getHostAddress();

		if (previousResponders.contains(localhost)) { return false; }
		header.addPreviousResponder(pInetAddress);
		return true;
	}

	protected static byte[] getBytes(ServiceLocationMessage pRequest, boolean pUsingMCast,
			boolean pUsingStreams) throws ServiceLocationException {

		ByteArrayOutputStream buff = new ByteArrayOutputStream();
		ServiceLocationHeader header = pRequest.getHeader();

		header.serialize(buff, pUsingMCast, pUsingStreams);
		byte _buff[] = buff.toByteArray();

		if (header.isOverflow()) throw new ServiceLocationException(
				ServiceLocationException.BUFFER_OVERFLOW);

		return _buff;
	}

	private static Socket getConnectionStream(InetAddress pInetAddress) throws IOException {

		Socket socket = null;
		if (socket == null) {
			socket = new Socket(pInetAddress, SLPDefaults.SLP_PORT);
			socket.setSoTimeout(SLPConfig.getGlobalSLPConfig().getTCPTimeout());
		}
		return socket;
	}

	private static int getNextTimeoutValue(long pInitialTime, long pCurrentMessageTime,
			int pMaxTimeOut, int[] pMsTimeOutsList) {
		int elapsedTime = (int) (pCurrentMessageTime - pInitialTime);

		if (pMaxTimeOut < elapsedTime) return -1;

		int acumulativeTime = 0;
		for (int i = 0; i < pMsTimeOutsList.length; i++) {

			acumulativeTime += pMsTimeOutsList[i];

			int tmpMaxTimeOut = acumulativeTime - elapsedTime;
			if (elapsedTime < acumulativeTime) {
				if (pMaxTimeOut - tmpMaxTimeOut < elapsedTime) { return pMaxTimeOut - elapsedTime; }
				return tmpMaxTimeOut;
			}
		}

		return -1;
	}

	protected static ServiceLocationMessage parseMsg(DataInputStream pInputStream)
			throws ServiceLocationException {
		// TODO: Implement Logging
		// if (SLPConfig.logger != null) {
		// SLPConfig.logger.entry(Level.INFO, MessageDispatcher.class,
		// "parseMsg");
		// }

		int version = 0;
		int functionCode = 0;
		ServiceLocationMessage response = null;
		byte tmpBuff[] = new byte[2];
		try {
			pInputStream.readFully(tmpBuff, 0, 2);
			version = tmpBuff[0] & 0xff;
			functionCode = tmpBuff[1] & 0xff;

			if (version != 2) throw new ServiceLocationException(
					ServiceLocationException.VERSION_NOT_SUPPORTED);

			SLPHeaderV2 header = new SLPHeaderV2();
			header.parseHeader(functionCode, pInputStream);

			// TODO: Implement Logging
			// if (SLPConfig.logger != null) {
			// SLPConfig.logger.text(Level.INFO, MessageDispatcher.class,
			// "parseMsg", "header="+header);
			// }

			if ((response = header.parseMsg(pInputStream)) != null) {
				header.parseOptions(pInputStream);
			}

		} catch (IllegalArgumentException e) {
			throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR, e);
		} catch (IOException e) {
			throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR, e);
		} catch (ServiceLocationException e) {
			throw e;
		}
		// TODO: Implement Logging
		// if (SLPConfig.logger != null) {
		// SLPConfig.logger.exit(Level.INFO, MessageDispatcher.class,
		// "parseMsg",response);
		// }

		return response;
	}

	public void run() {
		iExErr = dispatchDatagramMsg(iMsgOut, iResponseList, iTimeoutList, iMaxResults, iAddress,
				iJustFirstResult);
	}

	public static ServiceLocationException dispatchDatagramMsg(ServiceLocationMessage pRequest,
			Vector pResponseList, int[] pTimeoutList, int pMaxResults, InetAddress pInetAddress,
			boolean pJustFirstResult) {
		DatagramSocket datagramsocket = null;

		boolean usingBroadcast = SLPConfig.getGlobalSLPConfig().isBroadcastOnly();
		if (usingBroadcast) pInetAddress = SLPConfig.getBroadcastAddress();

		try {
			// InetAddress localhost = SLPConfig.getLocalHost();
			if (usingBroadcast) {
				try {
					datagramsocket = new DatagramSocket();
				} catch (SocketException e) {
					e.printStackTrace();
				}
			} else {
				MulticastSocket multicastsocket = null;
				try {
					multicastsocket = new MulticastSocket();
					multicastsocket.setTimeToLive(SLPConfig.getGlobalSLPConfig()
							.getMulticastRadius());
					// multicastsocket.setInterface(localhost);
				} catch (IOException e) {}
				datagramsocket = multicastsocket;
			}

			pResponseList.addAll(dispatchMulticastMsg(pInetAddress, datagramsocket, pRequest,
					pTimeoutList, pMaxResults, pJustFirstResult));

			datagramsocket.close();
			datagramsocket = null;
		} catch (ServiceLocationException e) {
			if (e.getErrorCode() != ServiceLocationException.DA_BUSY) { return e; }
		} catch (Exception e) {
			return new ServiceLocationException(ServiceLocationException.INTERNAL_SYSTEM_ERROR, e);
		} finally {
			if (datagramsocket != null) datagramsocket.close();
		}
		return null;
	}

	public static Vector dispatchMulticastMsg(InetAddress pInetAddress,
			DatagramSocket pDatagramSocket, ServiceLocationMessage pRequest, int[] pMsTimeOutsList,
			int pMaxResults, boolean pGetFirstResult) throws ServiceLocationException {

		Vector results = new Vector();
		int totalResponses = 0;
		int retry = 0;
		int maxTimeOut = SLPConfig.getGlobalSLPConfig().getMulticastMaximumWait();

		long initialTime = System.currentTimeMillis();
		int mtu = SLPConfig.getGlobalSLPConfig().getMTU();
		try {
			byte _request[] = getBytes(pRequest, true, false);
			pDatagramSocket.send(new DatagramPacket(_request, _request.length, pInetAddress, 427));

			retry++;
			long currentMessageTime = System.currentTimeMillis();
			while (totalResponses < pMaxResults) {

				byte tmpBuf[] = new byte[mtu];
				DatagramPacket datagrampacket = new DatagramPacket(tmpBuf, tmpBuf.length);

				int currentTimeoutvalue = getNextTimeoutValue(initialTime, currentMessageTime,
						maxTimeOut, pMsTimeOutsList);

				if (currentTimeoutvalue < 0) break; // we are done

				pDatagramSocket.setSoTimeout(currentTimeoutvalue);
				try {
					pDatagramSocket.receive(datagrampacket);
				} catch (InterruptedIOException e) {
					int elapsed = (int) (System.currentTimeMillis() - initialTime);
					if ((pGetFirstResult && totalResponses > 0) || elapsed > maxTimeOut
							|| retry >= pMsTimeOutsList.length) {
						System.out.println("getFirstResult:" + pGetFirstResult);
						System.out.println("totalResponses:" + totalResponses);
						System.out.println("elapsed:" + elapsed);
						System.out.println("maxTimeOut:" + maxTimeOut);
						System.out.println("retry:" + retry);
						System.out.println("msTimeOutsList.length:" + pMsTimeOutsList.length);
						break;
					}

					pDatagramSocket.send(new DatagramPacket(_request, _request.length,
							pInetAddress, 427));

					retry++;
					currentMessageTime = System.currentTimeMillis();
					continue;
				}

				// DataInputStream dis = new DataInputStream(new
				// DebugInputStream(new
				// ByteArrayInputStream(datagrampacket.getData()), System.out));
				DataInputStream dis = new DataInputStream(new ByteArrayInputStream(datagrampacket
						.getData()));
				InetAddress responderInetAddress = datagrampacket.getAddress();

				ServiceLocationMessage response = null;

				try {
					response = parseMsg(dis);
				} catch (ServiceLocationException e) {
					if (e.getErrorCode() == ServiceLocationException.PARSE_ERROR) {
						addPreviousResponder(pRequest, responderInetAddress);
						continue;
					}
					throw e;
				}

				if (pRequest.getHeader().getXid() != response.getHeader().getXid()
						|| !addPreviousResponder(pRequest, responderInetAddress)) continue;

				ServiceLocationHeader rspHeader = response.getHeader();
				if (rspHeader.isOverflow()) {
					response = dispatchStreamMsg(responderInetAddress, pRequest, false);
					if (response == null) continue;
					rspHeader = response.getHeader();
				}

				if (results.size() < pMaxResults) results.addElement(response);
				totalResponses += rspHeader.getNumberOfReplies();
				if (pGetFirstResult) break;
			}
		} catch (ServiceLocationException e) {
			if (e.getErrorCode() == ServiceLocationException.PREVIOUS_RESPONDER_OVERFLOW) { return results; }
			throw e;
		} catch (IOException ioexception) {
			throw new ServiceLocationException(ServiceLocationException.NETWORK_ERROR, ioexception);
		}

		return results;
	}

	private static ServiceLocationMessage dispatchUnicastMsg(DatagramSocket pSocket,
			InetAddress pInetAddress, ServiceLocationMessage pRequest)
			throws ServiceLocationException {

		ServiceLocationMessage rspMsg = null;
		byte _request[] = getBytes(pRequest, false, false);

		byte tmpBuff[] = new byte[8192];

		DatagramPacket inputDatagramPacket = new DatagramPacket(tmpBuff, tmpBuff.length);
		DatagramPacket outputDatagramPacket = new DatagramPacket(_request, _request.length,
				pInetAddress, 427);

		int timeoutList[] = SLPConfig.getGlobalSLPConfig().getDatagramTimeouts();
		for (int i = 0; i < timeoutList.length; i++)
			try {
				pSocket.setSoTimeout(timeoutList[i]);
				pSocket.send(outputDatagramPacket);
				pSocket.receive(inputDatagramPacket);
				DataInputStream datainputstream = new DataInputStream(new ByteArrayInputStream(
						inputDatagramPacket.getData()));
				rspMsg = parseMsg(datainputstream);
				break;
			} catch (InterruptedIOException e) {
				// connection time outs, try again
				// TODO convergence algorithm must be used here
			} catch (IOException e) {
				throw new ServiceLocationException(ServiceLocationException.NETWORK_ERROR);
			}

		if (rspMsg == null) { throw new ServiceLocationException(
				ServiceLocationException.NETWORK_TIMED_OUT); }
		return rspMsg;
	}

	public static ServiceLocationMessage dispatchStreamMsg(InetAddress pInetAddress,
			ServiceLocationMessage pRequest, boolean pRetry) throws ServiceLocationException {

		ServiceLocationMessage response = null;
		try {
			response = _dispatchStreamMsg(pInetAddress, pRequest, pRetry);
		} catch (InterruptedIOException e) {
			throw new ServiceLocationException(ServiceLocationException.NETWORK_TIMED_OUT, e);
		} catch (IOException e) {
			throw new ServiceLocationException(ServiceLocationException.NETWORK_ERROR, e);
		}
		if (pRequest.getHeader().getXid() != response.getHeader().getXid()) { return null; }
		return response;
	}

	private static ServiceLocationMessage _dispatchStreamMsg(InetAddress pInetAddress,
			ServiceLocationMessage pRequest, boolean pRetry) throws InterruptedIOException,
			IOException, ServiceLocationException {

		Socket socket = null;
		byte _request[] = getBytes(pRequest, false, true);
		try {
			socket = getConnectionStream(pInetAddress);
			DataOutputStream os = new DataOutputStream(socket.getOutputStream());
			DataInputStream is = new DataInputStream(socket.getInputStream());

			try {
				synchronized (socket) {
					os.write(_request, 0, _request.length);
					ServiceLocationMessage response = parseMsg(is);
					return response;
				}
			} catch (IOException e) {
				try {
					socket.close();
				} catch (IOException ee) {}
				socket = null;
				if (!pRetry) throw e;

				return _dispatchStreamMsg(pInetAddress, pRequest, false);
			}
		} finally {
			if (socket != null) {
				try {
					socket.close();
				} catch (IOException ee) {}
			}
		}
	}

	public static Vector dispatchUAMsg(Vector pDAList, ServiceLocationMessage pRequestMsg,
			InetAddress pInetAddress) throws ServiceLocationException {
		Vector resultSet = new Vector();
		ServiceLocationException daException = null;
		ServiceLocationException saException = null;

		// TODO follow the RFC 2608 sec 12.3, multicast must be used in case no
		// DA is present

		if (pDAList != null && pDAList.size() > 0) {
			daException = dispatchDAMsg(pDAList, pRequestMsg, resultSet, SLPConfig
					.getGlobalSLPConfig().getMaximumResults());

			if (daException != null && daException.getErrorCode() != ServiceLocationException.OK) dispatchDatagramMsg(
					pRequestMsg, resultSet, SLPConfig.getGlobalSLPConfig().getMulticastTimeouts(),
					SLPConfig.getGlobalSLPConfig().getMaximumResults(), pInetAddress, false);
		} else {
			saException = dispatchDatagramMsg(pRequestMsg, resultSet, SLPConfig
					.getGlobalSLPConfig().getMulticastTimeouts(), SLPConfig.getGlobalSLPConfig()
					.getMaximumResults(), pInetAddress, false);
		}

		if (resultSet.size() <= 0) {
			if (daException != null) {
				short errorCode = daException.getErrorCode();
				if (errorCode != ServiceLocationException.VERSION_NOT_SUPPORTED
						&& errorCode != ServiceLocationException.OK
						&& errorCode != ServiceLocationException.INTERNAL_ERROR
						&& errorCode != ServiceLocationException.OPTION_NOT_SUPPORTED
						&& errorCode != ServiceLocationException.REQUEST_NOT_SUPPORTED) throw daException;
			}
			if (saException != null) {
				short errorCode = saException.getErrorCode();
				if (errorCode != ServiceLocationException.VERSION_NOT_SUPPORTED
						&& errorCode != ServiceLocationException.OK
						&& errorCode != ServiceLocationException.INTERNAL_ERROR
						&& errorCode != ServiceLocationException.OPTION_NOT_SUPPORTED
						&& errorCode != ServiceLocationException.REQUEST_NOT_SUPPORTED) throw saException;
			}
		}
		return resultSet;
	}

	public static ServiceLocationException dispatchDAMsg(Vector pDAList,
			ServiceLocationMessage pRequest, Vector pResults, int pMaxReplies) {

		DatagramSocket datagramsocket = null;
		ServiceLocationException delayedException = null;
		InetAddress daInetAddress = null;
		int totalReplies = 0;

		try {
			if (datagramsocket == null) datagramsocket = new DatagramSocket();

			// ServiceLocationHeader srvlocheader = request.getHeader();

			Enumeration enumeration = pDAList.elements();
			while (enumeration.hasMoreElements()) {
				try {
					daInetAddress = (InetAddress) enumeration.nextElement();

					ServiceLocationMessage response = dispatchUnicastMsg(datagramsocket,
							daInetAddress, pRequest);

					if (pRequest.getHeader().getXid() != response.getHeader().getXid()) continue;

					ServiceLocationHeader rspHeader = response.getHeader();
					if (rspHeader.isOverflow()) {
						response = dispatchStreamMsg(daInetAddress, pRequest, false);
						if (response == null) continue;
					}

					rspHeader = response.getHeader();
					totalReplies += rspHeader.getNumberOfReplies();
					pResults.addElement(response);
					// break;
				} catch (ServiceLocationException e) {
					short errorCode = e.getErrorCode();
					if (errorCode != ServiceLocationException.DA_BUSY) delayedException = e;

					if (errorCode == ServiceLocationException.NETWORK_TIMED_OUT) {
						// DA not responding, remove it from the DA table
					}
				}
			}
			return new ServiceLocationException(ServiceLocationException.OK);
		} catch (SocketException e) {
			delayedException = new ServiceLocationException(ServiceLocationException.NETWORK_ERROR,
					e);
		} finally {
			if (datagramsocket != null) datagramsocket.close();
		}
		return delayedException;
	}

	public static byte[] getByteArray(String pHex) {
		byte[] buf = new byte[16];
		int total = 0;
		int pos = 0;
		while (pos + 2 < pHex.length()) {
			String val = null;
			val = pHex.substring(pos, pos + 2);
			byte b = Byte.parseByte(val, 16);
			buf[total++] = b;
			if (total >= buf.length) {
				byte[] tmp = new byte[buf.length << 1];
				System.arraycopy(buf, 0, tmp, 0, total);
				buf = tmp;
			}
			pos += 3;
		}
		byte[] tmp = new byte[total];
		System.arraycopy(buf, 0, tmp, 0, total);
		return tmp;
	}

	public static void main(String[] args) {
		System.setProperty("net.slp.traceMsg", "true");
		String bytes = "02 02 00 00 37 00 00 00 00 00 77 05 00 05 65 66 25 55 53 00 00 00 01 00 77 77 00 22 73 "
				+ "65 72 76 61 63 65 32 77 62 65 65 32 60 74 74 70 32 27 27 31 26 31 26 37 32 26 37 36 32 35 31 30 30 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 "
				+ "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 ";
		parseMsg(new DataInputStream(new ByteArrayInputStream(getByteArray(bytes))));
	}
}
