/**
 * SLPConfig.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 * ------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.sblim.slp.SLPConfigProperties;
import org.sblim.slp.ServiceLocationException;

public final class SLPConfig {

	private static InetAddress cBroadcastAddress;

	private static SLPConfig cInstance = null;

	private static InetAddress cLocalHostAddress;

	private static InetAddress cLoopBackAddress;

	private static InetAddress cMulticastAddress;

	static {
		try {
			cLoopBackAddress = InetAddress.getByName(SLPDefaults.LOOPBACK_ADDRESS);
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		try {
			cMulticastAddress = InetAddress.getByName(SLPDefaults.MULTICAST_ADDRESS);
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		try {
			cBroadcastAddress = InetAddress.getByName(SLPDefaults.BROADCAST_ADDRESS);
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		try {
			cLocalHostAddress = InetAddress.getLocalHost();
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
	}

	public static InetAddress getBroadcastAddress() {
		return cBroadcastAddress;
	}

	public static synchronized SLPConfig getGlobalSLPConfig() {
		if (cInstance == null) {
			cInstance = new SLPConfig();
		}
		return cInstance;
	}

	public static InetAddress getLoopback() {
		return cLoopBackAddress;
	}

	public static InetAddress getMulticastAddress() {
		return cMulticastAddress;
	}

	private static int getIntProperty(String pName, int pDefaultValue, int pMinValue, int pMaxValue) {
		int value = Integer.getInteger(pName, pDefaultValue).intValue();
		return min(pMaxValue, max(pMinValue, value));
	}

	private static int max(int pLhs, int pRhs) {
		return (pLhs < pRhs) ? pRhs : pLhs;
	}

	private static int min(int pLhs, int pRhs) {
		return (pLhs < pRhs) ? pLhs : pRhs;
	}

	private Class iCimClientProperties = null;

	private Logger iLogger = null;

	private SLPConfig() {

		try {
			// try to load config class from CIM client. This will cause load of
			// the CIM client's config file.
			iCimClientProperties = Class.forName("org.sblim.wbem.util.SessionProperties");
		} catch (ClassNotFoundException e) {}

		Properties slpProperties = new Properties(new Properties());
		try {
			InputStream inputstream = getConfigURLStream();
			if (inputstream != null) {
				slpProperties.load(inputstream);
			}
			for (Iterator iterator = slpProperties.entrySet().iterator(); iterator.hasNext();) {
				Map.Entry entry = (Map.Entry) iterator.next();
				System
						.setProperty(String.valueOf(entry.getKey()), String.valueOf(entry
								.getValue()));
			}
		} catch (IOException e) {
			System.out.println("Error while parsing property file");
			e.printStackTrace();
		}

		if (iCimClientProperties != null) {
			try {
				Method globalPropertiesGetter = iCimClientProperties.getMethod(
						"getGlobalProperties", null);
				Object globalProperties = globalPropertiesGetter.invoke(null, null);
				Method getLogger = iCimClientProperties.getMethod("getLogger", null);
				iLogger = (Logger) getLogger.invoke(globalProperties, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		if (iLogger == null) {
			iLogger = Logger.getLogger("org.sblim.slp");
			iLogger.setUseParentHandlers(false);
			try {
				// TODO get log file name from properties
				FileHandler fileHandler = new FileHandler(SLPDefaults.LOG_FILE);
				fileHandler.setLevel(Level.INFO);
				iLogger.addHandler(fileHandler);
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	public int getActiveDiscoveryGranularity() {
		return getIntProperty(SLPConfigProperties.NET_SLP_DA_ACTIVE_DISCOVERY_GRANULARITY,
				SLPDefaults.ACTIVE_DISCOVERY_GRANULARITY, SLPLimits.MIN_DISCOVERY_GRANULARITY,
				SLPLimits.MAX_DISCOVERY_GRANULARITY);
	}

	public int getActiveDiscoveryInterval() {
		int interval = getIntProperty(SLPConfigProperties.NET_SLP_DA_ACTIVE_DISCOVERY_INTERVAL,
				SLPDefaults.ACTIVE_DISCOVERY_INTERVAL, SLPLimits.MIN_DISCOVERY_INTERVAL,
				SLPLimits.MAX_DISCOVERY_INTERVAL); // 3mins
		if (interval > 0 && interval < 300) { return 300; // prevent network
		// fludding
		}
		return interval;
	}

	public int[] getDADiscoveryTimeouts() {
		return parseTimeouts(SLPConfigProperties.NET_SLP_DA_DISCOVERY_TIMEOUTS,
				SLPDefaults.DA_DISCOVERY_TIMEOUTS);
	}

	public int[] getDatagramTimeouts() {
		return parseTimeouts(SLPConfigProperties.NET_SLP_DATAGRAM_TIMEOUTS,
				SLPDefaults.DATAGRAM_TIMEOUTS);
	}

	public Vector getInterfaces() {

		String property = System.getProperty(SLPConfigProperties.NET_SLP_INTERFACES);

		List addresses = new LinkedList();

		if (property != null && property.trim().length() > 0) {
			try {
				addresses.addAll(Utils.parseCommaSeparatedList(property, false));
			} catch (ServiceLocationException e) {
				// TODO logging
			}
		}

		final int count = addresses.size();
		List interfaces = new LinkedList();

		if (count == 0) {
			interfaces.add(cLocalHostAddress);
			interfaces.add(cLoopBackAddress);
		} else {
			for (int i = 0; i < count; ++i) {
				String address = (String) addresses.get(i);
				try {
					InetAddress inetAddress = InetAddress.getByName(address);
					if (!interfaces.contains(inetAddress)) {
						if (inetAddress.equals(cLocalHostAddress)) {
							interfaces.add(0, inetAddress);
						} else {
							interfaces.add(inetAddress);
						}
					}
				} catch (UnknownHostException e) {
					// TODO log this
					continue;
				}
			}
		}
		return new Vector(interfaces);
	}

	public InetAddress getLocalHost() {
		List interfaces = getInterfaces();
		return (InetAddress) interfaces.get(0);
	}

	public int getMaximumResults() {
		int value = Integer.getInteger(SLPConfigProperties.NET_SLP_MAX_RESULTS,
				SLPDefaults.MAXIMUM_RESULTS).intValue();
		return (value >= 1 && value <= SLPDefaults.MAXIMUM_RESULTS) ? value
				: SLPDefaults.MAXIMUM_RESULTS;
	}

	public int getMTU() {
		return getIntProperty(SLPConfigProperties.NET_SLP_MTU, SLPDefaults.MTU, SLPLimits.MIN_MTU,
				SLPLimits.MAX_MTU);
	}

	public int getMulticastMaximumWait() {
		return getIntProperty(SLPConfigProperties.NET_SLP_MULTICAST_MAXIMUM_WAIT,
				SLPDefaults.MULTICAST_MAXIMUM_WAIT, SLPLimits.MIN_MULTICAST_MAXIMUM_WAIT,
				SLPLimits.MAX_MULTICAST_MAXIMUM_WAIT);
	}

	public int getMulticastRadius() {
		return getIntProperty(SLPConfigProperties.NET_SLP_MULTICAST_TTL,
				SLPDefaults.MULTICAST_RADIUS, SLPLimits.MIN_MULTICAST_RADIUS,
				SLPLimits.MAX_MULTICAST_RADIUS);
	}

	public int[] getMulticastTimeouts() {
		return parseTimeouts(SLPConfigProperties.NET_SLP_MULTICAST_TIMEOUTS,
				SLPDefaults.MULTICAST_TIMEOUTS);
	}

	public Vector getPreconfiguredDAs() {

		String addressString = System.getProperty(SLPConfigProperties.NET_SLP_DA_ADDRESSES, "");
		if (addressString != null && addressString.trim().length() > 0) {
			try {
				List addresses = Utils.parseCommaSeparatedList(addressString, true);
				Vector result = new Vector();
				for (Iterator iter = addresses.iterator(); iter.hasNext();) {
					try {
						result.add(InetAddress.getByName((String) iter.next()));
					} catch (UnknownHostException e) {
						// TODO: log this
					}
				}
				return result;
			} catch (ServiceLocationException e) {}
		}
		return new Vector();
	}

	public Vector getConfiguredScopes() {
		Vector scopes = parseScopes(SLPConfigProperties.NET_SLP_USE_SCOPES);
		if (scopes.isEmpty()) {
			scopes.add(SLPDefaults.DEFAULT_SCOPE);
		}
		return scopes;
	}

	public Vector getSAOnlyScopes() {
		return parseScopes(SLPConfigProperties.NET_SLP_SAONLY_SCOPES);
	}

	public int getServerSocketQueueLength() {
		return getIntProperty(SLPConfigProperties.NET_SLP_SERVER_SOCKET_QUEUE_LENGTH,
				SLPDefaults.SERVER_SOCKET_QUEUE_LENGTH, SLPLimits.MIN_SERVER_SOCKET_QUEUE_LENGTH,
				SLPLimits.MAX_SERVER_SOCKET_QUEUE_LENGTH);
	}

	public int getTCPTimeout() {
		return getIntProperty(SLPConfigProperties.NET_SLP_TCPTIMEOUT, SLPDefaults.TCP_TIMEOUT,
				SLPLimits.MIN_TCP_TIMEOUT, SLPLimits.MAX_TCP_TIMEOUT);
	}

	public boolean getTraceMsg() {
		return Boolean.getBoolean(SLPConfigProperties.NET_SLP_TRACE_MSG);
	}

	public boolean isBroadcastOnly() {
		return Boolean.getBoolean(SLPConfigProperties.NET_SLP_IS_BROADCAST_ONLY);
	}

	public boolean isDA() {
		return Boolean.getBoolean(SLPConfigProperties.NET_SLP_IS_DA);
	}

	public boolean isSA() {
		return !isDA();
	}

	private InputStream getConfigURLStream() {

		String configURL = System.getProperty(SLPConfigProperties.NET_SLP_CONFIG_URL);

		if (configURL != null) {
			if (configURL.trim().length() > 0) {
				try {
					URL url = new URL(configURL);
					InputStream inputstream = url.openStream();
					return inputstream;
				} catch (MalformedURLException e) {
					// TODO log this
				} catch (IOException e) {
					// TODO LOG this... config file not found
				}
			}
			return null;
		}
		for (int i = 0; i < SLPDefaults.CONF_URLS.length; ++i) {
			configURL = SLPDefaults.CONF_URLS[i];
			try {
				URL url = new URL(configURL);
				InputStream inputstream = url.openStream();
				return inputstream;
			} catch (Exception e) {}
		}
		return null;
	}

	private Vector parseScopes(String pPropertyName) {

		String propertyValue = System.getProperty(pPropertyName);
		if (propertyValue != null && propertyValue.length() > 0) {
			try {
				Vector result = Utils.parseCommaSeparatedList(propertyValue, true);
				Utils.unescapeScopeStrings(result);
				// TODO Validate scopes
				return result;
			} catch (ServiceLocationException e) {
				e.printStackTrace();
			}
		}
		return new Vector();
	}

	private int[] parseTimeouts(String pPropertyName, int[] pDefaultTimeouts) {
		String value = System.getProperty(pPropertyName);

		if (value == null || value.length() <= 0) return pDefaultTimeouts;

		Vector list = null;
		try {
			list = Utils.parseCommaSeparatedList(value, true);
		} catch (ServiceLocationException e) {
			return pDefaultTimeouts;
		}

		int timeouts[] = new int[list.size()];
		int pos = 0;
		for (Enumeration enumeration = list.elements(); enumeration.hasMoreElements();) {
			String timeoutElem = (String) enumeration.nextElement();
			try {
				timeouts[pos++] = Integer.parseInt(timeoutElem);
			} catch (NumberFormatException e) {
				return pDefaultTimeouts;
			}
		}
		return timeouts;
	}
}
