/**
 * SLPHeaderV2.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationAttribute;
import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceURL;

class SLPHeaderV2 extends ServiceLocationHeader implements Cloneable {

	protected static Hashtable optClasses = new Hashtable();

	private int extOff;

	// private Hashtable optTable;

	private static final int MAX_MESSAGE_LENGTH = 0xffffff;

	protected static final int FLAG_BYTE = 4;

	protected static final int NOFLAG = 0;

	protected static final int OVERFLOW = 128;

	protected static final int FRESH = 64;

	protected static final int MCAST = 32;

	protected static final int HEADER_SIZE = 14;

	public SLPHeaderV2() {
		extOff = 0;
		super.setVersion(2);
	}

	public SLPHeaderV2(int pFunctionCode, boolean pFresh, Locale pLocale)
			throws ServiceLocationException {
		extOff = 0;

		super.setVersion(2);
		super.setFunctionCode(pFunctionCode);
		super.setLocale(pLocale);
		super.setXid(ServiceLocationHeader.getUniqueXID());
		super.setFresh(pFresh);

		String lagTag = pLocale.getCountry();
		lagTag = pLocale.getLanguage() + (lagTag.length() > 0 ? "-" + lagTag : "");

		byte _lagTag[] = Utils.getStringBytes(lagTag, "UTF8");
		int headerSize = _lagTag.length + HEADER_SIZE;

		if (super.getPacketLength() - headerSize < 2) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR);

	}

	public Object clone() throws CloneNotSupportedException {
		SLPHeaderV2 slpheaderv2 = (SLPHeaderV2) super.clone();
		byte _lanTag[] = Utils.getStringBytes(super.getLocale().toString(), "UTF8");
		slpheaderv2.setNumberOfBytes(HEADER_SIZE + _lanTag.length + 2);
		return slpheaderv2;
	}

	public void serialize(ByteArrayOutputStream pStream, boolean pUsingMCast, boolean pUsingStreams)
			throws ServiceLocationException {

		super.setMCast(pUsingMCast);

		String lagTag = super.getLocale().getCountry();
		lagTag = super.getLocale().getLanguage() + (lagTag.length() > 0 ? "-" + lagTag : "");

		byte _lagTag[] = Utils.getStringBytes(lagTag, "UTF8");
		int lagTagSize = _lagTag.length;

		ByteArrayOutputStream extensionsOutputStream = new ByteArrayOutputStream();
		if (super.getFunctionCode() == SRVACK || super.getFunctionCode() == SRVTYPERPLY
				|| super.getFunctionCode() == SRVRPLY || super.getFunctionCode() == ATTRRPLY
				|| super.getFunctionCode() == DAADVERT) putInt(super.getErrorCode(),
				extensionsOutputStream);

		int availableSize = super.getPacketLength()
				- (super.getPayload().length + HEADER_SIZE + lagTagSize);

		Vector previousRespondersList = super.getPreviousResponse();
		if (previousRespondersList != null) {
			previousRespondersList = pUsingMCast ? previousRespondersList : new Vector();
			serializePreviousResponders(previousRespondersList, extensionsOutputStream,
					availableSize);
		}
		if (super.getErrorCode() == 0) {
			extensionsOutputStream.write(super.getPayload(), 0, super.getPayload().length);
			// serialize options
			// extOff = serializeOptions(extensionsOutputStream, lagTagSize);
		}
		byte _options[] = extensionsOutputStream.toByteArray();
		super.setLength(HEADER_SIZE + lagTagSize + _options.length);

		if (super.getLength() > MAX_MESSAGE_LENGTH) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR, "size overflow - maximum size reached");

		if (!pUsingStreams && super.getLength() > super.getPacketLength()) {
			super.setOverflow(true);
			super.setLength(super.getPacketLength());
			byte _msg[] = new byte[super.getPacketLength()];
			System.arraycopy(_options, 0, _msg, 0, super.getLength() - (HEADER_SIZE + lagTagSize));
			_options = _msg;
		}
		pStream.write((byte) (0xff & super.getVersion()));
		pStream.write((byte) (0xff & super.getFunctionCode()));
		putInt24(super.getLength(), pStream);

		byte flags = NOFLAG;

		if (super.isOverflow()) flags |= OVERFLOW;
		if (super.isFresh()) flags = (byte) ((flags | FRESH) & 0xFF);
		if (pUsingMCast) flags = (byte) ((flags | MCAST) & 0xFF);

		// flags
		pStream.write((byte) (0xFF & flags));
		// reserved
		pStream.write(0);

		putInt24(extOff, pStream);
		putInt(super.getXid(), pStream);
		putInt(lagTagSize, pStream);
		pStream.write(_lagTag, 0, _lagTag.length);
		pStream.write(_options, 0, _options.length);
	}

	public void parseHeader(int pFunctionCode, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		super.setFunctionCode(pFunctionCode);
		super.incrementNumberOfBytes(2);
		super.setLength(getInt24(pStream));
		byte buf[] = new byte[2];
		pStream.readFully(buf, 0, 2);
		super.incrementNumberOfBytes(2);

		byte flags = (byte) (buf[0] & 0xff);
		super.setOverflow((flags & OVERFLOW) != 0);
		super.setFresh((flags & FRESH) != 0);
		super.setMCast((flags & MCAST) != 0);

		extOff = getInt24(pStream);
		if (extOff <= super.getLength()) {
			super.setXid((short) getInt(pStream));
			StringBuffer stringbuffer = new StringBuffer();
			getString(stringbuffer, pStream);

			String lanTag = stringbuffer.toString();
			int i = lanTag.indexOf('-');
			if (i > -1) {
				super.setLocale(new Locale(lanTag.substring(0, i), lanTag.substring(i + 1)));
			} else super.setLocale(new Locale(lanTag, ""));

			super.setErrorCode((short) 0);
		} else throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR);
	}

	public ServiceLocationMessage parseMsg(DataInputStream pStream)
			throws ServiceLocationException, IOException {
		// TODO: Implement Logging
		// if (SLPConfig.logger != null) {
		// SLPConfig.logger.entry(Level.INFO, SLPHeaderV2.class,
		// "parseMsg");
		// }
		ServiceLocationMessage msg = null;
		if (super.getFunctionCode() != SAADVERT) super.setErrorCode((short) getInt(pStream));

		switch (super.getFunctionCode()) {
			case SRVRPLY:
				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.text(Level.INFO, SLPHeaderV2.class,
				// "parseMsg", "SLPServiceReplyMessage()");
				// }
				msg = new SLPServiceReplyMessage(this, pStream);
				break;
			case SRVACK:
				super.setNumberOfReplies(1);
				msg = this;
				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.text(Level.INFO, SLPHeaderV2.class,
				// "parseMsg", "SLPAck()");
				// }
				break;
			case ATTRRPLY:
				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.message(Level.INFO, SLPHeaderV2.class,
				// "parseMsg", "SLPAttributeReplyMessage()");
				// }
				msg = new SLPAttributeReplyMessage(this, pStream);
				break;
			case SRVTYPERPLY:
				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.message(Level.INFO, SLPHeaderV2.class,
				// "parseMsg", "SLPServiceTypeReplyMessage()");
				// }
				msg = new SLPServiceTypeReplyMessage(this, pStream);
				break;
			case DAADVERT:
				// // TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.message(Level.INFO, SLPHeaderV2.class,
				// "parseMsg", "SLPDAAdvert()");
				// }
				msg = new SLPDAAdvert(this, pStream);
				break;
			case SAADVERT:
				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.message(Level.INFO, SLPHeaderV2.class,
				// "parseMsg", "SLPSAAdvert()");
				// }
				msg = new SLPSAAdvert(this, pStream);
				break;
			default:
				throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR);
		}
		if (super.getNumberOfBytes() > super.getLength()) { throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR); }
		return msg;
	}

	ServiceURL parseServiceURL(DataInputStream pStream, short pErrorCode)
			throws ServiceLocationException, IOException {

		// reserved
		byte buf[] = new byte[1];
		pStream.readFully(buf, 0, 1);

		super.incrementNumberOfBytes(1);
		int lifetime = getInt(pStream);

		StringBuffer sb = new StringBuffer();
		byte _url[] = getString(sb, pStream);

		buf = new byte[1];
		pStream.readFully(buf, 0, 1);
		super.incrementNumberOfBytes(1);
		byte numOfAuths = (byte) (buf[0] & 0xff);
		if (numOfAuths > 0) {
			// TODO check auth block list
			// update the lifetime from the auth table
			// throw new
			// ServiceLocationException(ServiceLocationException.AUTHENTICATION_FAILED);
		}
		String s = sb.toString();
		ServiceURL serviceurl = null;
		try {
			serviceurl = new ServiceURL(s, lifetime);
		} catch (IllegalArgumentException e) {
			throw new ServiceLocationException(pErrorCode, e);
		}

		return serviceurl;
	}

	public void parseOptions(DataInputStream pStream) throws ServiceLocationException, IOException,
			IllegalArgumentException {
		if (extOff == 0) {
			// TODO: Implement Logging
			// if (SLPConfig.logger != null) {
			// SLPConfig.logger.text(Level.INFO, ServiceLocationHeader.class,
			// "parseOptions", "parsing options...");
			// }
			return;
		}
		// TODO: Implement Logging
		// if (SLPConfig.logger != null) {
		// SLPConfig.logger.text(Level.INFO, ServiceLocationHeader.class,
		// "parseOptions", "Warning: Parsing Options is not implemented on this
		// release");
		// }
	}

	private void serializePreviousResponders(Vector pPreviousRespondersList,
			ByteArrayOutputStream pStream, int pMaxSize) throws ServiceLocationException {
		ByteArrayOutputStream prevRspBuf = new ByteArrayOutputStream();
		int size = pPreviousRespondersList.size();
		for (int i = 0; i < size; i++) {
			String s = (String) pPreviousRespondersList.elementAt(i);
			if (i > 0) s = "," + s;
			byte _prevRsp[] = Utils.getStringBytes(s, "UTF8");
			if (_prevRsp.length <= pMaxSize) {
				prevRspBuf.write(_prevRsp, 0, _prevRsp.length);
				pMaxSize -= _prevRsp.length;
			} else {
				throw new ServiceLocationException(
						ServiceLocationException.PREVIOUS_RESPONDER_OVERFLOW,
						"prev responders overflow");
			}
		}

		byte _previousRespondersList[] = prevRspBuf.toByteArray();
		putInt(_previousRespondersList.length, pStream);
		pStream.write(_previousRespondersList, 0, _previousRespondersList.length);
		super.incrementNumberOfBytes(_previousRespondersList.length);
	}

	public boolean serializeServiceURL(ServiceURL pServiceUrl, Hashtable pSpiAuthBlock,
			ByteArrayOutputStream pStream, boolean pUseMcast) throws ServiceLocationException {

		// boolean useAuthBlock = spiAuthBlock != null;

		ByteArrayOutputStream bytearrayoutputstream1 = new ByteArrayOutputStream();
		int initialSize = super.getNumberOfBytes();
		byte _serviceurl[] = Utils.getStringBytes(pServiceUrl.toString(), "UTF8");
		bytearrayoutputstream1.write(0);
		super.incrementNumberOfBytes(1);
		putInt(pServiceUrl.getLifetime(), bytearrayoutputstream1);
		byte authBlockSize = 0;
		// if (useAuthBlock) {
		// if (spiAuthBlock == null) {
		// ByteArrayOutputStream bytearrayoutputstream2 = new
		// ByteArrayOutputStream();
		// ServiceLocationHeader.putInteger(_serviceurl.length,
		// bytearrayoutputstream2);
		// Object urlAuthBlockPair[] = new Object[2];
		// urlAuthBlockPair[0] = bytearrayoutputstream2.toByteArray();
		// urlAuthBlockPair[1] = _serviceurl;
		// spiAuthBlock = getCheckedAuthBlockList(urlAuthBlockPair,
		// serviceurl.getLifetime());
		// }
		// authBlockSize = (byte)spiAuthBlock.size();
		// Object aobj[] = AuthBlock.getContents(spiAuthBlock);
		// _serviceurl = (byte[])aobj[1];
		// }
		putInt(_serviceurl.length, bytearrayoutputstream1);
		bytearrayoutputstream1.write(_serviceurl, 0, _serviceurl.length);
		super.incrementNumberOfBytes(_serviceurl.length);
		bytearrayoutputstream1.write((byte) (0xff & authBlockSize));
		super.incrementNumberOfBytes(1);
		// if (authBlockSize > 0)
		// AuthBlock.externalizeAll(this, spiAuthBlock, bytearrayoutputstream1);
		_serviceurl = bytearrayoutputstream1.toByteArray();

		if (!pUseMcast || super.getNumberOfBytes() <= super.getPacketLength()) {
			pStream.write(_serviceurl, 0, _serviceurl.length);
			return true;
		}
		super.setNumberOfBytes(initialSize); // restore the size, since
												// message
		// overflows
		return false;
	}

	public void parseAuthenticatedAttributeVector(Vector pVector, DataInputStream pStream,
			boolean pFlag) throws ServiceLocationException, IOException {

		byte _attributeList[] = parseAttributeVector(pVector, pStream);

		ByteArrayOutputStream tmpBuf = new ByteArrayOutputStream();
		ServiceLocationHeader.putInteger(_attributeList.length, tmpBuf);

		byte _numOfAttrAuth[] = new byte[1];
		pStream.readFully(_numOfAttrAuth, 0, 1);
		super.incrementNumberOfBytes(1);
		byte numOfAttrAuth = (byte) (_numOfAttrAuth[0] & 0xff);
		if (numOfAttrAuth > 0) {
			// TODO check auth
		}
		return;

	}

	public byte[] parseAttributeVector(Vector pVector, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		StringBuffer sb = new StringBuffer();
		byte buf[] = getString(sb, pStream);
		Vector attributeElements = Utils.parseCommaSeparatedList(sb.toString(), false);
		int j = attributeElements.size();
		for (int i = 0; i < j; i++) {
			String s = (String) attributeElements.elementAt(i);
			pVector.addElement(new ServiceLocationAttribute(s));
		}

		return buf;
	}

	/**
	 * 
	 * @param pElements
	 * @param pStream
	 * @throws ServiceLocationException
	 * @throws IOException
	 */
	void parseSignatureAndValidateAttributes(Vector pElements, DataInputStream pStream)
			throws ServiceLocationException, IOException {
	// TODO this should return the AuthBlock

	}

	public byte[] serializeAttributeVector(Vector pAttributes, ByteArrayOutputStream pStream,
			boolean pHasSecurity) throws ServiceLocationException {
		byte _attrListStr[] = null;
		int authBlockSize = 0;

		Vector attrList = new Vector();

		ServiceLocationAttribute srvlocattr;
		Enumeration enumeration = pAttributes.elements();
		while (enumeration.hasMoreElements()) {
			srvlocattr = (ServiceLocationAttribute) enumeration.nextElement();
			attrList.addElement(srvlocattr.serialize());
		}

		String attrListStr = Utils.vectorToCommaSeparatedList(attrList);
		_attrListStr = Utils.getStringBytes(attrListStr, "UTF8");

		putInt(_attrListStr.length, pStream);
		pStream.write(_attrListStr, 0, _attrListStr.length);
		super.incrementNumberOfBytes(_attrListStr.length);

		if (pHasSecurity) {
			pStream.write((byte) (authBlockSize & 0xff));
			super.incrementNumberOfBytes(1);
		}
		// TODO serialize all authBlock!
		return _attrListStr;
	}

	public boolean serializeServiceURL(ServiceURL pServiceUrl, ByteArrayOutputStream pStream,
			boolean pUsingDatagram) throws ServiceLocationException {

		ByteArrayOutputStream tmpBuff = new ByteArrayOutputStream();
		int i = super.getNumberOfBytes();
		byte _serviceurl[] = Utils.getStringBytes(pServiceUrl.toString(), "UTF8");

		tmpBuff.write(0);
		super.incrementNumberOfBytes(1);
		putInt(pServiceUrl.getLifetime(), tmpBuff);
		byte numOfAuthBlocks = 0;

		putInt(_serviceurl.length, tmpBuff);
		tmpBuff.write(_serviceurl, 0, _serviceurl.length);
		super.incrementNumberOfBytes(_serviceurl.length);

		tmpBuff.write((byte) (0xff & numOfAuthBlocks));
		super.incrementNumberOfBytes(1);

		// TODO serialize authorization blocks

		_serviceurl = tmpBuff.toByteArray();

		if (!pUsingDatagram || super.getNumberOfBytes() <= super.getPacketLength()) {
			pStream.write(_serviceurl, 0, _serviceurl.length);
			return true;
		}
		super.setNumberOfBytes(i);
		return false;
	}

}
