/**
 * SLPRequestHandler.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;

public class SLPRequestHandler implements Runnable {

	private InetAddress iInterface;

	private Socket iSocket;

	private DatagramPacket iPacket;

	private InetAddress iClientAddr;

	private int iPort;

	private ServiceTable iServiceTable;

	private DataInputStream iInputStream;

	private DataOutputStream iOutputStream;

	private static Hashtable cInProgressMap = new Hashtable();

	public SLPRequestHandler(DatagramPacket pDatagramPacket, InetAddress pInetAddress) {
		iPort = 0;
		iPacket = pDatagramPacket;
		iClientAddr = iPacket.getAddress();
		iPort = iPacket.getPort();
		iInterface = pInetAddress;
		iServiceTable = new ServiceTable();
	}

	public SLPRequestHandler(Socket pSocket, InetAddress pInetAddress) {
		iPort = 0;
		iSocket = pSocket;
		iClientAddr = iSocket.getInetAddress();
		iPort = iSocket.getPort();
		iInterface = pInetAddress;
		iServiceTable = new ServiceTable();
	}

	public void run() {
		if (iSocket != null || iInputStream != null) {
			try {
				// TODO get the timeout time from the config

				iSocket.setSoTimeout(SLPConfig.getGlobalSLPConfig().getTCPTimeout());
			} catch (SocketException e) {
				// TODO Log
				e.printStackTrace();
			}
			try {
				boolean closeConnection;
				do {
					iInputStream = new DataInputStream(iSocket.getInputStream());
					iOutputStream = new DataOutputStream(iSocket.getOutputStream());

					ByteArrayOutputStream tmpOutputStream = new ByteArrayOutputStream();
					closeConnection = handleRequest(iInputStream, tmpOutputStream, true);
					iOutputStream.write(tmpOutputStream.toByteArray(), 0, tmpOutputStream.size());
					// forward messages to DA
				} while (!closeConnection);
			} catch (EOFException e) {
				// e.printStackTrace();
			} catch (IOException e) {
				// e.printStackTrace();
			}
			try {
				if (iSocket != null) iSocket.close();
				iSocket = null;
			} catch (IOException e) {
				// TODO log
				// e.printStackTrace();
			}
		} else {
			byte xidArray[] = new byte[2];

			byte data[] = iPacket.getData();
			System.arraycopy(data, 10, xidArray, 0, 2);
			int XID;
			XID = (xidArray[0] & 0xFF) << 8;
			XID |= xidArray[1] & 0xFF;
			String requestID = XID + ":" + iClientAddr.getHostAddress();

			// System.out.println("Address: "+packet.getAddress());
			// System.out.println("Socket Address: "+packet.getSocketAddress());

			boolean processingRequest = false;
			synchronized (cInProgressMap) {
				processingRequest = cInProgressMap.get(requestID) != null;
				if (!processingRequest) cInProgressMap.put(requestID, this);
			}
			if (processingRequest) {
				// request already in progress, ignore it
				return;
			}
			DataInputStream datainputstream = new DataInputStream(new ByteArrayInputStream(data));
			ByteArrayOutputStream tmpOutputStream = new ByteArrayOutputStream();
			try {
				handleRequest(datainputstream, tmpOutputStream, false);
				byte responseArray[] = tmpOutputStream.toByteArray();
				if (responseArray != null && responseArray.length > 0) {
					DatagramSocket datagramsocket = null;
					try {
						datagramsocket = new DatagramSocket();
						DatagramPacket datagrampacket = new DatagramPacket(responseArray,
								responseArray.length, iClientAddr, iPort);
						datagramsocket.send(datagrampacket);
						// TODO forward the message to the DA
					} catch (SocketException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					} finally {
						if (datagramsocket != null) datagramsocket.close();
					}
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
			synchronized (cInProgressMap) {
				SLPRequestHandler requesthandler = (SLPRequestHandler) cInProgressMap
						.get(requestID);
				if (requesthandler == this) cInProgressMap.remove(requestID);
			}
		}
	}

	private boolean handleRequest(DataInputStream pRsqInputStream,
			ByteArrayOutputStream pRspOutputStream, boolean pUsingStreams) throws IOException {
		boolean closeConnection = false;

		ServiceLocationMessage rsq = parseRequest(pRsqInputStream, pUsingStreams);
		ServiceLocationMessage rsp = rsq;

		if (rsq != null) {
			ServiceLocationHeader srvlocheader = rsq.getHeader();
			if (srvlocheader.getErrorCode() == 0) { // if header was parsed
				// successfully

				if (isInResponderList(srvlocheader)) return false;

				switch (rsq.getHeader().getFunctionCode()) {
					case ServiceLocationHeader.SRVRQST:
						rsp = dispatchSrvRqst((SLPServiceRequestMessage) rsq);
						break;
					case ServiceLocationHeader.SRVREG:
						rsp = dispatchReg((SLPServiceRegistration) rsq);
						break;
					case ServiceLocationHeader.SRVDEREG:
						rsp = dispatchDereg((SLPServiceDeregistration) rsq);
						break;
					case ServiceLocationHeader.ATTRRQST:
						rsp = null; // TODO
						break;
					case ServiceLocationHeader.SRVTYPERQST:
						rsp = dispatchSrvType((SLPServiceTypeRequestMessage) rsq);
						break;
					case ServiceLocationHeader.SAADVERT:
						rsp = null; // TODO
						break;
					case ServiceLocationHeader.DAADVERT:
						rsp = null; // TODO
						break;
					default:
						rsp = null;
				}

				if (rsp == null) { return false; // keep alive
				}
			} else if (rsq.getHeader().isMCast()) { // if its a multicast
				// message
				// don't responde
				rsp = null;
			} else closeConnection = pUsingStreams; // retry only if using
			// streams
		}

		if (rsp != null) {
			ServiceLocationHeader rspHdr = rsp.getHeader();
			try {
				rspHdr.serialize(pRspOutputStream, false, pUsingStreams);
			} catch (ServiceLocationException servicelocationexception) {
				ServiceLocationMessage reply = rspHdr.makeErrorReply(servicelocationexception);

				rspHdr = reply.getHeader();
				try {
					pRspOutputStream.reset();
					rspHdr.serialize(pRspOutputStream, false, pUsingStreams);
				} catch (ServiceLocationException e) {
					e.printStackTrace();
				}
			}
		} else {
			// System.out.println("null message");
		}
		return closeConnection;
	}

	// private void sendDatagramReply(byte msg[]) {
	// }

	private ServiceLocationMessage parseRequest(DataInputStream pStream, boolean pUsingStreams)
			throws IOException {
		int version = 0;
		int functionID = 0;
		try {
			byte buff[] = new byte[2];
			pStream.readFully(buff, 0, 2);
			version = (char) buff[0] & 0xFF;
			functionID = (char) buff[1] & 0xFF;
		} catch (IOException e) {
			throw e;
		}
		ServiceLocationMessage srvlocmsg = null;
		SLPServerHeaderV2 header = null;
		try {
			if (version != 2) throw new ServiceLocationException(
					ServiceLocationException.VERSION_NOT_SUPPORTED);

			// header = new SLPHeaderV2();
			header = new SLPServerHeaderV2();

			if (pUsingStreams) header.setPacketLength(0x7FFFFFFF); // max size
																	// for the
																	// response

			header.parseHeader(functionID, pStream);

			srvlocmsg = header.parseMsg(pStream);
			if (srvlocmsg != null) header.parseOptions(pStream);

		} catch (Exception e) {
			e.printStackTrace();
			srvlocmsg = null;
			if (functionID != ServiceLocationHeader.DAADVERT
					&& functionID != ServiceLocationHeader.SAADVERT && header != null) srvlocmsg = header
					.makeErrorReply(e);
		}
		return srvlocmsg;
	}

	public boolean isInResponderList(ServiceLocationHeader pHeader) {
		if (pHeader.getPreviousResponse() == null || !pHeader.isMCast()) return false;
		Vector vector = pHeader.getPreviousResponse();

		List interfaces = SLPConfig.getGlobalSLPConfig().getInterfaces();
		Enumeration enumeration = vector.elements();
		while (enumeration.hasMoreElements()) {
			try {
				String responderAddress = (String) enumeration.nextElement();
				InetAddress inetaddress = InetAddress.getByName(responderAddress);
				if (interfaces.contains(inetaddress)) return true;
			} catch (UnknownHostException e) {
				// TODO can not resolve the element
				e.printStackTrace();
			}
		}
		return false;
	}

	private ServiceLocationMessage dispatchSrvRqst(SLPServiceRequestMessage pRequestMsg) {
		ServiceLocationHeader srvlocheader = pRequestMsg.getHeader();
		boolean mcast = srvlocheader.isMCast();
		String serviceType = pRequestMsg.getServiceType().toString();
		ServiceLocationMessage responseMsg = null;

		if (serviceType.equals(SLPDefaults.DA_SERVICE_TYPE.toString())) {
			if (SLPConfig.getGlobalSLPConfig().isDA()) {
				// responseMsg = serviceTable.makeDAAdvert(ssrvmsg, interfac);
				srvlocheader = responseMsg.getHeader();
			}
			if (srvlocheader.getErrorCode() != 0 && mcast) { return null; }
			return responseMsg;
		}
		if (serviceType.equals(SLPDefaults.SA_SERVICE_TYPE.toString())) {
			if (!mcast) return null; // drop if not multicast

			try {
				responseMsg = iServiceTable.makeSAAdvert(pRequestMsg, iInterface);
			} catch (ServiceLocationException e) {
				e.printStackTrace();
			}
			return responseMsg;
		}
		// DA service:directory-agent

		if (mcast && SLPConfig.getGlobalSLPConfig().isDA()) { // if this is a DA ignore multicast
			// messages
			return null;
		}

		responseMsg = iServiceTable.findServices(pRequestMsg);
		srvlocheader = responseMsg.getHeader();
		if (mcast && (srvlocheader.getErrorCode() != 0 || srvlocheader.getNumberOfReplies() == 0)) { return null; }
		return responseMsg;
	}

	private ServiceLocationMessage dispatchReg(SLPServiceRegistration pMsg) {
		ServiceLocationHeader srvlocheader = pMsg.getHeader();
		if (srvlocheader.isMCast()) { return null; }
		ServiceLocationMessage srvlocmsg = iServiceTable.register(pMsg);

		if (srvlocmsg != null) {
			ServiceLocationHeader header = srvlocmsg.getHeader();
			if (header.getErrorCode() == 0) {
				// TODO if successfully completed, then forward it to the DA
			}

		}
		return srvlocmsg;
	}

	private ServiceLocationMessage dispatchDereg(SLPServiceDeregistration pMsg) {
		ServiceLocationHeader srvlocheader = pMsg.getHeader();
		if (srvlocheader.isMCast()) { return null; }
		ServiceLocationMessage srvlocmsg = iServiceTable.deregister(pMsg);

		if (srvlocmsg != null) {
			ServiceLocationHeader header = srvlocmsg.getHeader();
			if (header.getErrorCode() == 0) {
				// TODO if successfully completed, then forward it to the DA
			}
		}
		return srvlocmsg;
	}

	private ServiceLocationMessage dispatchSrvType(SLPServiceTypeRequestMessage pRst) {
		ServiceLocationHeader srvlocheader = pRst.getHeader();
		if (srvlocheader.isMCast()) { return null; }
		ServiceLocationMessage rsp = iServiceTable.findServiceTypes(pRst);

		if (rsp != null) {
			ServiceLocationHeader header = rsp.getHeader();
			if (header.getErrorCode() == 0) {
				// TODO if successfully completed, then forward it to the DA
			}
		}
		return rsp;
	}

}
