/**
 * SLPServerHeaderV2.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Locale;

import org.sblim.slp.ServiceLocationException;

class SLPServerHeaderV2 extends SLPHeaderV2 implements Cloneable {

	private int iReplyFunctionCode;

	public SLPServerHeaderV2() {
		iReplyFunctionCode = SRVACK;
	}

	public SLPServerHeaderV2(int pFunctionCode, boolean pFlag, Locale pLocale)
			throws ServiceLocationException {
		super(pFunctionCode, pFlag, pLocale);
		iReplyFunctionCode = SRVACK;
	}

	public ServiceLocationMessage makeErrorReply(Exception pReason) {
		ServiceLocationHeader srvlocheader = null;
		try {
			srvlocheader = (ServiceLocationHeader) clone();
		} catch (CloneNotSupportedException e) {}

		srvlocheader.setFresh(false);
		srvlocheader.setOverflow(false);
		srvlocheader.setFunctionCode(iReplyFunctionCode);

		if (pReason instanceof ServiceLocationException) {
			srvlocheader.setErrorCode(((ServiceLocationException) pReason).getErrorCode());
			if (srvlocheader.getErrorCode() >= 0 && srvlocheader.getErrorCode() <= 22) srvlocheader
					.setErrorCode(ServiceLocationException.INTERNAL_ERROR);
		} else if ((pReason instanceof IllegalArgumentException)
				|| (pReason instanceof IOException)) srvlocheader
				.setErrorCode(ServiceLocationException.PARSE_ERROR);
		else srvlocheader.setErrorCode(ServiceLocationException.INTERNAL_ERROR);

		return srvlocheader;
	}

	public SLPServerHeaderV2 makeReplyHeader() {
		SLPServerHeaderV2 that = null;
		try {
			that = (SLPServerHeaderV2) clone();
		} catch (CloneNotSupportedException e) {}
		that.setFunctionCode(iReplyFunctionCode);
		that.setLength(0);
		that.setPreviousResponse(null);
		that.setScopes(null);
		that.setOverflow(false);
		that.setFresh(false);
		that.setMCast(false);
		return that;
	}

	public void parseHeader(int pFunctionCode, DataInputStream pStream)
			throws ServiceLocationException, IOException {
		switch (pFunctionCode) {
			case SRVRQST:
				iReplyFunctionCode = SRVRPLY;
				break;
			case ATTRRQST:
				iReplyFunctionCode = ATTRRPLY;
				break;
			case SRVTYPERQST:
				iReplyFunctionCode = SRVTYPERPLY;
				break;
			case SRVREG:
			case SRVDEREG:
				iReplyFunctionCode = SRVACK;
				break;
			case DAADVERT:
				iReplyFunctionCode = DAADVERT;
				break;
		}
		super.parseHeader(pFunctionCode, pStream);
	}

	public ServiceLocationMessage parseMsg(DataInputStream pStream)
			throws ServiceLocationException, IOException, IllegalArgumentException {

		ServiceLocationMessage msg = null;
		if (super.getFunctionCode() == DAADVERT) super.setErrorCode((short) getInt(pStream));

		switch (super.getFunctionCode()) {
			case SRVRQST:
				msg = new SLPServiceRequestMessage(this, pStream);
				break;
			case SRVREG:
				msg = new SLPServiceRegistration(this, pStream);
				break;
			case SRVDEREG:
				msg = new SLPServiceDeregistration(this, pStream);
				break;
			case ATTRRQST:
				msg = new SLPAttributeRequestMessage(this, pStream);
				break;
			case SRVACK:
				msg = this;
				super.setNumberOfReplies(1);
				break;
			case SRVTYPERQST:
				msg = new SLPServiceTypeRequestMessage(this, pStream);
				break;
			case DAADVERT:
				msg = new SLPDAAdvert(this, pStream);
				break;
			case SAADVERT:
				msg = new SLPSAAdvert(this, pStream);
				break;
			default:
				return null;
		}
		if (super.getNumberOfBytes() > super.getLength()) { throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR); }
		return (msg);
	}

	public void parsePreviousResponders(DataInputStream pStream) throws ServiceLocationException,
			IOException {

		StringBuffer stringbuffer = new StringBuffer();
		getString(stringbuffer, pStream);
		super.setPreviousResponse(Utils.parseCommaSeparatedList(stringbuffer.toString(), true));
	}

	public void parseScopes(DataInputStream pStream) throws ServiceLocationException, IOException {

		StringBuffer stringbuffer = new StringBuffer();
		getString(stringbuffer, pStream);

		super.setScopes(Utils.parseCommaSeparatedList(stringbuffer.toString(), true));
		Utils.unescapeScopeStrings(super.getScopes());
		// Validate scopes
		// System.out.println("scopes needs to be validated");
	}

}
