/**
 * ServiceLocationHeader.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.util.Locale;
import java.util.Random;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;

public abstract class ServiceLocationHeader implements ServiceLocationMessage, Cloneable {

	public static final int ATTRRPLY = 7;

	public static final int ATTRRQST = 6;

	private static short cUniqueXID = 0;

	public static final int DAADVERT = 8;

	public static final int SAADVERT = 11;

	public static final int SRVACK = 5;

	public static final int SRVDEREG = 4;

	public static final int SRVREG = 3;

	public static final int SRVRPLY = 2;

	public static final int SRVRQST = 1;

	public static final int SRVTYPERPLY = 10;

	public static final int SRVTYPERQST = 9;

	public static String getBytesString(byte[] pBuffer, String pEncoding) {
		try {
			return new String(pBuffer, pEncoding);
		} catch (UnsupportedEncodingException e) {
			return "";
		}
	}

	public static int getInteger(DataInputStream pStream) throws ServiceLocationException,
			IOException {
		byte buf[] = new byte[2];
		pStream.readFully(buf, 0, 2);
		int b0 = buf[0] & 0xff;
		int b1 = buf[1] & 0xff;
		int res = b0 << 8;
		res += b1;
		return res;
	}

	public static byte[] getStringField(StringBuffer pBuffer, DataInputStream pStream,
			String pString) throws ServiceLocationException, IOException {
		pBuffer.setLength(0);
		int size = 0;
		size = getInteger(pStream);
		byte buf[] = new byte[size];
		pStream.readFully(buf, 0, size);
		pBuffer.append(getBytesString(buf, pString));
		return buf;
	}

	public static synchronized short getUniqueXID() {
		if (cUniqueXID == 0) {
			Random random = new Random();
			cUniqueXID = (short) (random.nextInt() & 0xFFFF);
		}
		cUniqueXID++;
		return (short) (cUniqueXID & 0xFFFF);
	}

	public static void putInteger(int pValue, ByteArrayOutputStream pStream) {
		pStream.write((byte) ((0xff00 & pValue) >> 8));
		pStream.write((byte) (0xff & pValue));
	}

	public static byte[] putStringField(String pString, ByteArrayOutputStream pStream,
			String pEncoding) {
		byte buf[] = Utils.getStringBytes(pString, pEncoding);
		putInteger(buf.length, pStream);
		pStream.write(buf, 0, buf.length);
		return buf;
	}

	private short iErrorCode = 0;

	private boolean iFresh = false;

	private int iFunctionCode = 0;

	private int iLength = 0;

	private Locale iLocale = SLPDefaults.LOCALE;

	private boolean iMCast = false;

	private int iNumberOfBytes = 0;

	private int iNumberOfReplies = 0;

	private boolean iOverflow = false;

	private int iPacketLength = SLPConfig.getGlobalSLPConfig().getMTU();

	private byte[] iPayload = new byte[0];

	private Vector iPreviousResponse = null;

	private Vector iScopes = null;

	private int iVersion = 0;

	private short iXid = 0;

	public ServiceLocationHeader() {}

	/**
	 * Getter for errorCode
	 * 
	 * @return Returns the errorCode.
	 */
	public short getErrorCode() {
		return iErrorCode;
	}

	/**
	 * Setter for errorCode
	 * 
	 * @param pErrorCode
	 *            The errorCode to set.
	 */
	public void setErrorCode(short pErrorCode) {
		iErrorCode = pErrorCode;
	}

	/**
	 * Getter for fresh
	 * 
	 * @return Returns the fresh.
	 */
	public boolean isFresh() {
		return iFresh;
	}

	/**
	 * Setter for fresh
	 * 
	 * @param pFresh
	 *            The fresh to set.
	 */
	public void setFresh(boolean pFresh) {
		iFresh = pFresh;
	}

	/**
	 * Getter for functionCode
	 * 
	 * @return Returns the functionCode.
	 */
	public int getFunctionCode() {
		return iFunctionCode;
	}

	/**
	 * Setter for functionCode
	 * 
	 * @param pFunctionCode
	 *            The functionCode to set.
	 */
	public void setFunctionCode(int pFunctionCode) {
		iFunctionCode = pFunctionCode;
	}

	/**
	 * Getter for length
	 * 
	 * @return Returns the length.
	 */
	public int getLength() {
		return iLength;
	}

	/**
	 * Setter for length
	 * 
	 * @param pLength
	 *            The length to set.
	 */
	public void setLength(int pLength) {
		iLength = pLength;
	}

	/**
	 * Getter for locale
	 * 
	 * @return Returns the locale.
	 */
	public Locale getLocale() {
		return iLocale;
	}

	/**
	 * Setter for locale
	 * 
	 * @param pLocale
	 *            The locale to set.
	 */
	public void setLocale(Locale pLocale) {
		iLocale = pLocale;
	}

	/**
	 * Getter for mCast
	 * 
	 * @return Returns the mCast.
	 */
	public boolean isMCast() {
		return iMCast;
	}

	/**
	 * Setter for mCast
	 * 
	 * @param pCast
	 *            The mCast to set.
	 */
	public void setMCast(boolean pCast) {
		iMCast = pCast;
	}

	/**
	 * Getter for numberOfBytes
	 * 
	 * @return Returns the numberOfBytes.
	 */
	public int getNumberOfBytes() {
		return iNumberOfBytes;
	}

	/**
	 * Setter for numberOfBytes
	 * 
	 * @param pNumberOfBytes
	 *            The numberOfBytes to set.
	 */
	public void setNumberOfBytes(int pNumberOfBytes) {
		iNumberOfBytes = pNumberOfBytes;
	}

	/**
	 * Getter for numberOfReplies
	 * 
	 * @return Returns the numberOfReplies.
	 */
	public int getNumberOfReplies() {
		return iNumberOfReplies;
	}

	/**
	 * Setter for numberOfReplies
	 * 
	 * @param pNumberOfReplies
	 *            The numberOfReplies to set.
	 */
	public void setNumberOfReplies(int pNumberOfReplies) {
		iNumberOfReplies = pNumberOfReplies;
	}

	/**
	 * Getter for overflow
	 * 
	 * @return Returns the overflow.
	 */
	public boolean isOverflow() {
		return iOverflow;
	}

	/**
	 * Setter for overflow
	 * 
	 * @param pOverflow
	 *            The overflow to set.
	 */
	public void setOverflow(boolean pOverflow) {
		iOverflow = pOverflow;
	}

	/**
	 * Getter for packetLength
	 * 
	 * @return Returns the packetLength.
	 */
	public int getPacketLength() {
		return iPacketLength;
	}

	/**
	 * Setter for packetLength
	 * 
	 * @param pPacketLength
	 *            The packetLength to set.
	 */
	public void setPacketLength(int pPacketLength) {
		iPacketLength = pPacketLength;
	}

	/**
	 * Getter for payload
	 * 
	 * @return Returns the payload.
	 */
	public byte[] getPayload() {
		return iPayload;
	}

	/**
	 * Setter for payload
	 * 
	 * @param pPayload
	 *            The payload to set.
	 */
	public void setPayload(byte[] pPayload) {
		iPayload = pPayload;
	}

	/**
	 * Getter for previousResponse
	 * 
	 * @return Returns the previousResponse.
	 */
	public Vector getPreviousResponse() {
		return iPreviousResponse;
	}

	/**
	 * Setter for previousResponse
	 * 
	 * @param pPreviousResponse
	 *            The previousResponse to set.
	 */
	public void setPreviousResponse(Vector pPreviousResponse) {
		iPreviousResponse = pPreviousResponse;
	}

	/**
	 * Getter for scopes
	 * 
	 * @return Returns the scopes.
	 */
	public Vector getScopes() {
		return iScopes;
	}

	/**
	 * Setter for scopes
	 * 
	 * @param pScopes
	 *            The scopes to set.
	 */
	public void setScopes(Vector pScopes) {
		iScopes = pScopes;
	}

	/**
	 * Getter for version
	 * 
	 * @return Returns the version.
	 */
	public int getVersion() {
		return iVersion;
	}

	/**
	 * Setter for version
	 * 
	 * @param pVersion
	 *            The version to set.
	 */
	public void setVersion(int pVersion) {
		iVersion = pVersion;
	}

	/**
	 * Getter for xid
	 * 
	 * @return Returns the xid.
	 */
	public short getXid() {
		return iXid;
	}

	/**
	 * Setter for xid
	 * 
	 * @param pXid
	 *            The xid to set.
	 */
	public void setXid(short pXid) {
		iXid = pXid;
	}

	public void addPreviousResponder(InetAddress pInetAddress) {
		String hostname = pInetAddress.getHostAddress();

		if (!iPreviousResponse.contains(hostname)) iPreviousResponse.addElement(hostname);
	}

	public Object clone() throws CloneNotSupportedException {
		ServiceLocationHeader srvlocheader = (ServiceLocationHeader) super.clone();
		srvlocheader.iLength = 0;
		srvlocheader.iPayload = new byte[0];
		srvlocheader.iNumberOfReplies = 0;
		return srvlocheader;
	}

	public ServiceLocationHeader getHeader() {
		return this;
	}

	public int getInt(DataInputStream pStream) throws ServiceLocationException, IOException {
		int i = getInteger(pStream);
		iNumberOfBytes += 2;
		return i;
	}

	protected int getInt24(DataInputStream pStream) throws ServiceLocationException, IOException {
		byte buf[] = new byte[3];
		pStream.readFully(buf, 0, 3);
		int b0 = buf[0] & 0xff;
		int b1 = buf[1] & 0xff;
		int b2 = buf[2] & 0xff;
		int res = b0 << 16;
		res += b1 << 8;
		res += b2;
		iNumberOfBytes += 3;
		return res;
	}

	public byte[] getString(StringBuffer pBuffer, DataInputStream pStream)
			throws ServiceLocationException, IOException {
		byte buf[] = getStringField(pBuffer, pStream, "UTF8");
		iNumberOfBytes += buf.length + 2;
		return buf;
	}

	public void incrementNumberOfBytes(int pIncrement) {
		iNumberOfBytes += pIncrement;
	}

	public ServiceLocationMessage makeErrorReply(Exception pException) {
		return null;
	}

	public abstract void parseHeader(int pFunctionCode, DataInputStream pStream)
			throws ServiceLocationException, IOException, IllegalArgumentException;

	public abstract ServiceLocationMessage parseMsg(DataInputStream pStream)
			throws ServiceLocationException, IOException, IllegalArgumentException;

	public abstract void parseOptions(DataInputStream pStream) throws ServiceLocationException,
			IOException, IllegalArgumentException;

	public void putInt(int pValue, ByteArrayOutputStream pStream) {
		putInteger(pValue, pStream);
		iNumberOfBytes += 2;
	}

	protected void putInt24(int pValue, ByteArrayOutputStream pStream) {
		pStream.write((byte) 0xff & (pValue >> 16));
		pStream.write((byte) 0xff & (pValue >> 8));
		pStream.write((byte) 0xff & (pValue));
		iNumberOfBytes += 3;
	}

	public byte[] putString(String pValue, ByteArrayOutputStream pStream) {
		byte buf[] = putStringField(pValue, pStream, "UTF8");
		iNumberOfBytes += buf.length + 2;
		return buf;
	}

	public abstract void serialize(ByteArrayOutputStream pStream, boolean pUsingMCast,
			boolean pUsingStreams) throws ServiceLocationException;

	protected byte[] serializeCommaSeparatedList(Vector pList, ByteArrayOutputStream pStream) {
		return putString(Utils.vectorToCommaSeparatedList(pList), pStream);
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("ServiceLocationHeader[");
		sb.append("errCode=" + iErrorCode);
		sb.append(",xid=" + iXid);
		sb.append(",version=" + iVersion);
		sb.append(",funcCode=" + iFunctionCode);
		sb.append(",length=" + iLength);
		sb.append(",overflow=" + iOverflow);
		sb.append(",fresh=" + iFresh);
		sb.append(",mcast=" + iMCast);
		sb.append(",locale=" + iLocale);
		sb.append(",numReplies=" + iNumberOfReplies);
		sb.append(",packetLength=" + iPacketLength);
		sb.append(",scopes=" + iScopes);
		sb.append("]");

		return sb.toString();
	}

}
