/**
 * ThreadPool.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.util.Vector;

public class ThreadPool {

	private Vector iIdleThreads = new Vector();

	protected Vector iThreadPool = new Vector();

	private Vector iQueue = new Vector();

	private static ThreadGroup cGroup = new ThreadGroup("TreadPool Group");

	private long iIdleTimeout = 5000;

	private int iMaxPoolSize = 5;

	private int iMinPoolSize = 2;

	// private int iThreadCounter = 0;

	// private int iPriority = Thread.NORM_PRIORITY;

	private int iCounter = 0;

	private static ThreadPool cInstance = new ThreadPool(2, 5);

	public synchronized static ThreadPool getInstance() {
		return cInstance;
	}

	private ThreadPool(int pMinPoolSize, int pMaxPoolSize) {
		this.iMinPoolSize = pMinPoolSize;
		this.iMaxPoolSize = pMaxPoolSize;

		for (int i = 0; i < pMinPoolSize; i++) {
			Worker worker = new Worker(this, "WorkerThread " + ((getID()) + 1));
			// idleThreads.add(worker);
			iThreadPool.add(worker);
			worker.start();
		}
	}

	private int getID() {
		iCounter++;
		if (iCounter > 1000000) iCounter = 0;
		return iCounter;
	}

	public synchronized long getIdleTimeOutMs() {
		// return (threadPool.size() <= idleThreads.size())? -1: idleTimeout;
		return (iIdleThreads.size() <= iMinPoolSize) ? -1 : iIdleTimeout;
	}

	public boolean execute(Runnable task) {
		return execute(task, true);
	}

	public synchronized boolean execute(Runnable pTask, boolean pEnqueue) {
		int totalIdle = iIdleThreads.size();
		if (totalIdle > 0) {
			Worker worker = (Worker) iIdleThreads.remove(totalIdle - 1);
			return worker.assignTask(pTask);
		} else if ((iMaxPoolSize == -1) || (iThreadPool.size() <= iMaxPoolSize)) {
			Worker worker = createWorker();
			return worker.assignTask(pTask);
		} else if (pEnqueue) {
			iQueue.add(pTask);
		} else {
			return false;
		}
		return true;
	}

	private synchronized Worker createWorker() {
		Worker worker = new Worker(this, "WorkerThread " + (getID() + 1));
		iThreadPool.add(worker);
		return worker;
	}

	public synchronized ThreadGroup getGroup() {
		return cGroup;
	}

	public synchronized boolean taskCompleted(Worker pWorker, boolean pTimeout) {
		if (iQueue.size() > 0) {
			iIdleThreads.remove(pWorker);
			pWorker.assignTask((Runnable) iQueue.remove(0));
			return true;
		} else if (pTimeout && (iThreadPool.size() > iMinPoolSize)) {
			iIdleThreads.remove(pWorker);
			iThreadPool.remove(pWorker);
			return false;
		} else if ((iMaxPoolSize == -1) || (iThreadPool.size() <= iMaxPoolSize)) {
			if (!iIdleThreads.contains(pWorker)) iIdleThreads.add(pWorker);
			return true;
		} else {
			return false;
		}
	}

	protected synchronized void removeThread(Worker pWorker) {
		if (pWorker != null) iThreadPool.remove(pWorker);
	}

	private class Worker extends Thread {

		private boolean iAlive = true;

		private boolean iStarted;

		private ThreadPool iPool;

		private Runnable iTask;

		public Worker(ThreadPool pPool) {
			this(pPool, "WorkerThread " + (iThreadPool.size() + 1));
		}

		public Worker(ThreadPool pPool, String pName) {
			super(pPool.getGroup(), pName);
			this.iPool = pPool;
			setDaemon(true);
		}

		public synchronized void start() {
			iStarted = true;
			iAlive = true;
			super.start();
		}

		public synchronized void kill() {
			iAlive = false;
			iStarted = false;
			notify();
		}

		public synchronized boolean assignTask(Runnable pTask) {
			if (iAlive) {
				this.iTask = pTask;
				if (!iStarted) start();
				notify();
				return true;
			}
			return false;
		}

		private synchronized Runnable waitForTask() {
			Runnable tsk;
			boolean timedout = false;
			while (iAlive) {
				if (iTask != null) {
					tsk = iTask;
					iTask = null;
					return tsk;
				}
				if ((iAlive = iPool.taskCompleted(this, timedout)) && (iTask == null)) {
					try {
						long idleTimeOut = iPool.getIdleTimeOutMs();
						timedout = false;
						if (idleTimeOut > 0) {
							wait(idleTimeOut);
							timedout = (iTask == null);
						} else {
							wait();
						}
					} catch (InterruptedException e) {
						// this is expected
					}
				}
			}
			return null;
		}

		public void run() {
			while (iAlive) {
				Runnable tsk = waitForTask();
				if (tsk != null) {
					try {
						tsk.run();
					} catch (Throwable t) {
						t.printStackTrace();
					}
				}
				if (!iAlive) break;
			}
			iStarted = false;
			iPool.removeThread(this);
		}
	}
}
