/**
 * CIMNameSpace.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * 
 * Change History
 * Flag     Date         Prog         Description
 *------------------------------------------------------------------------------- 
 * 17997    08/04/2005   thschaef     Log right class (this)
 * 18076    08/11/2005   pineiro5     localhost incorrectly added to CIMObjectPath
 * 1535756   2006-08-08  lupusalex    Make code warning free
 */

package org.sblim.wbem.cim;

import java.io.Serializable;
import java.net.URI;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.sblim.wbem.util.GlobalProperties;

public class CIMNameSpace implements Serializable, Cloneable {

	private static final long serialVersionUID = 4090304642015551161L;

	private static final String HTTP = "http";

	// private static final String HTTPS = "https";

	private static final String DEFAULT_PROTOCOL = HTTP;

	private static final String DEFAULT_FILE = "/cimom";

	private static final String DEFAULT_HOST = null;

	private static final int DEFAULT_PORT = 5988;

	private static final String DEFAULT_NAMESPACE = "root/cimv2";

	public static final char NAMESPACE_SEPARATOR = '/';

	private String iHost;

	private String iNamespace;

	private int iPort;

	private String iFile = "/";

	private URI iUri;

	private String iProtocol = DEFAULT_PROTOCOL;

	/**
	 * Constructs and instantiates a default CIM namespace name.
	 * 
	 * @throws CIMException
	 *             CIM_ERR_FAILED - if a malformed URL is constructed from the
	 *             specified parameters
	 */
	public CIMNameSpace() throws CIMException {
		iHost = DEFAULT_HOST;
		iPort = DEFAULT_PORT;
		iFile = DEFAULT_FILE;
		iProtocol = DEFAULT_PROTOCOL;
		iNamespace = DEFAULT_NAMESPACE;

		try {
			iUri = new URI(iProtocol, null, iHost, iPort, iFile, null, null);
		} catch (Exception e) {
			throw new CIMException(CIMException.CIM_ERR_FAILED, e);
		}
	}

	/**
	 * Constructs a CIM namespace, pointing to a specified host or URI
	 * (protocol://hostname[:port]/file), i.e.
	 * "https://47.11.8.15:5989/root/cimv2"
	 * 
	 * @param pURI
	 *            The URI String
	 * @throws CIMException
	 */
	public CIMNameSpace(String pURI) throws CIMException {
		// TODO validate NameSpace
		this(pURI, null);
	}

	/**
	 * Constructs an object which represents a CIMNameSpace.
	 * 
	 * @param pHost
	 *            host= protocol://hostname[:port]/file host=
	 *            "https://myhostname/" host= "http://myhostname:5988/cimom"
	 *            host= "myhostname"
	 * @param pNamespace
	 *            a string which represents the namespace in the CIM Object
	 *            Manager. According to the spec it should not starts or ends
	 *            with '/'. This are examples of valid namespace. namespace=
	 *            "root/cimv2" namespace= "root"
	 * @throws IllegalArgumentException
	 *             if the
	 * @throws CIMException
	 *             CIM_ERR_INVALID_PARAMETER - if the URI is malformed
	 */
	public CIMNameSpace(String pHost, String pNamespace) throws CIMException {
		iUri = null;
		if (pHost == null) pHost = DEFAULT_HOST;
		if (pNamespace == null) pNamespace = DEFAULT_NAMESPACE;
		setNamespace(pNamespace);

		try {
			if (pHost.indexOf('/') == -1) {
				int pos = pHost.indexOf(':');
				if (pos == -1 && (pos + 1) == pHost.length()) // does not
				// contain a
				// port
				iUri = new URI(DEFAULT_PROTOCOL, null, pHost, DEFAULT_PORT, DEFAULT_FILE, null,
						null);
				else iUri = new URI(DEFAULT_PROTOCOL, pHost, DEFAULT_FILE, null);
			} else {
				iUri = new URI(pHost);
			}
		} catch (Exception e) {
			throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER, e);
		}
		iHost = iUri.getHost();
		iPort = iUri.getPort();
		iFile = iUri.getPath();
		iProtocol = iUri.getScheme();
		if (iFile == null || iFile.length() == 0 || iFile.equals("/")) iFile = DEFAULT_FILE;
		if (iPort == -1) iPort = DEFAULT_PORT;
		// TODO validate NameSpace
		iNamespace = pNamespace;
		if (iHost == null) throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
				"Invalid hostname: \"" + pHost + "\"");
		try {
			iUri = new URI(iProtocol, null, iHost, iPort, iFile, null, null);
		} catch (Exception e) {
			Logger logger = GlobalProperties.getLogger();
			if (logger.isLoggable(Level.WARNING)) {
				logger.log(Level.WARNING, "invalid URI format", e);
			}
			throw new CIMException(CIMException.CIM_ERR_FAILED, e, "malformed URI");
		}
	}

	protected void updateURI() throws CIMException {
		try {
			iUri = new URI(iProtocol, null, iHost, iPort, iFile, null, null);
		} catch (Exception e) {
			// uri = null;

			Logger logger = GlobalProperties.getLogger();
			if (logger.isLoggable(Level.WARNING)) {
				logger.log(Level.WARNING, "invalid URI format", e);
			}
			throw new CIMException(CIMException.CIM_ERR_FAILED, e, "malformed URI");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
		CIMNameSpace that = new CIMNameSpace();
		that.iHost = iHost;
		that.iNamespace = iNamespace;
		that.iPort = iPort;
		that.iProtocol = iProtocol;
		that.iFile = iFile;
		that.iUri = iUri;
		return that;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 * 
	 * Compares a namespace with the given object. It is important to notice
	 * that the hostname must be exactly the same (ignoring case). The IP
	 * address of the hostname name is not resolved.
	 * 
	 * @return true if the namespace value for this namespace is equal to the
	 * namespace value for the given object and both URI matches.
	 */
	public boolean equals(Object o) {
		if (!(o instanceof CIMNameSpace)) return false;
		CIMNameSpace that = (CIMNameSpace) o;

		if (!iUri.equals(that.iUri)) return false;

		return iNamespace.equals(that.iNamespace);
	}

	/**
	 * Returns the hostname of the CIMNameSpace
	 * 
	 * @return a string which represents the hostname of the NameSpace
	 */
	public String getHost() {
		return iHost;
	}

	/**
	 * Returns the URI representation of the CIMNameSpace.
	 * 
	 * @return The URI
	 * @throws CIMException
	 *             malformed URI exception
	 */
	public URI getHostURI() throws CIMException {
		return iUri;
	}

	/**
	 * Returns the URL representation of the CIMNameSpace.
	 * 
	 * @return The URL
	 * @throws CIMException
	 *             malformed URL exception
	 * @see java.net.URI#toURL()
	 */
	public URL getHostURL() throws CIMException {
		URL url = null;
		try {
			url = iUri.toURL();
		} catch (Throwable e) {
			Logger logger = GlobalProperties.getLogger();
			if (logger.isLoggable(Level.WARNING)) {
				logger.log(Level.WARNING, "invalid URI format", e);
			}
			try {
				url = new URL(iProtocol, iHost, iPort, iFile);
			} catch (Throwable n) {
				// url = null;

				if (logger.isLoggable(Level.WARNING)) {
					logger.log(Level.WARNING, "invalid URI format", e);
				}
				throw new CIMException(CIMException.CIM_ERR_FAILED, n, "malformed URI");
			}
		}
		return url;
	}

	/**
	 * Returns the namespace fragment of the CIMNameSpace. This fragment does
	 * not include any information related to a host. (i.e. "myhost:root/cimv2",
	 * this method return just the "root/cimv2"
	 * 
	 * @return The namespace
	 */
	public String getNameSpace() {
		return iNamespace;
	}

	/**
	 * Returns the scheme specified on the URI.
	 * 
	 * @return The scheme
	 */
	public String getScheme() {
		return iUri.getScheme();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return iUri.hashCode();
	}

	/**
	 * Creates a CIMNameSpace object from the specified URI. This method behaves
	 * as new CIMNameSpace(p).
	 * 
	 * @param pURI
	 *            The URI string
	 * @return The namespace
	 * @throws CIMException
	 * 
	 * @deprecated The CIMNameSpace(String) constructor must be used instead of
	 *             this method.
	 */
	public static CIMNameSpace parse(String pURI) throws CIMException {
		return new CIMNameSpace(pURI);
	}

	/**
	 * Specifies the port for this CIMNameSpace object.
	 * 
	 * @param pPort
	 */
	public void setPort(int pPort) {
		if (pPort < 0) throw new IllegalArgumentException("invalid port number argument: " + pPort);
		iPort = pPort;
		updateURI();
	}

	/**
	 * Returns the port defined for this namespace by the URI. If no port is
	 * defined, then the default port is used.
	 * 
	 * @return The port
	 */
	public int getPort() {
		return iPort;
	}

	/**
	 * Specifies the host for this CIMNameSpace object.
	 * 
	 * @param pHost
	 */
	public void setHost(String pHost) {
		if (pHost == null) throw new IllegalArgumentException("null hostname argument");
		String realHost = pHost;
		int i;
		if ((i = pHost.indexOf(':')) > -1) {
			realHost = pHost.substring(0, i);
			String realPort = pHost.substring(i + 1);

			int newPort = -1;
			try {
				newPort = Integer.parseInt(realPort);
				iPort = newPort;
			} catch (NumberFormatException e) {
				newPort = -1;
			}
			iHost = realHost;
		} else iHost = pHost;
		updateURI();
	}

	/**
	 * Specifies the namespace for this CIMNameSpace object. The namespace
	 * should not start or end with a '/', it will be automatically removed. As
	 * a result a namespace like "/root/cimv2", will result on "root/cimv2", and
	 * "/" would be translated into an empty namespace ("")
	 * 
	 * @param pNamespace
	 *            The namespace
	 * @throws CIMException
	 */
	public void setNameSpace(String pNamespace) throws CIMException {
		if (pNamespace == null) throw new IllegalArgumentException("null namespace argument");
		setNamespace(pNamespace);
		updateURI();
	}

	protected void setNamespace(String namespace) {
		if (namespace != null) {
			if (namespace.indexOf('\\') > -1) throw new CIMException(
					CIMException.CIM_ERR_INVALID_PARAMETER,
					"namespace should not contain '\\' character");
			boolean tryAgain = false;
			do {
				tryAgain = false;
				if (namespace.startsWith("/")) {
					namespace = namespace.substring(1);
					tryAgain = true;
				}
				if (namespace.endsWith("/")) {
					namespace = namespace.substring(0, namespace.length() - 1);
					tryAgain = true;
				}
			} while (tryAgain);
			iNamespace = namespace;
		} else iNamespace = "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "//" + ((iHost == null) ? "" : iHost) + '/'
				+ ((iNamespace == null) ? "" : iNamespace) + "/";
	}
}
